#include "benondelettes.h"

/*  benondelettes.cpp  
    adapt  partir de : http://www.epm6604b.be/biblio/biblio.html
                         http://www.epm6604b.be/biblio/detail_ondelette.html
*/

/*

  Bibliothque de transforme en ondelette d'un signal.

  Cette bibliothque dfinit:

  - la transforme et la transforme inverse de l'ondelette de Haar;
  - la transforme normalise et la transforme inverse normalise de l'ondelette de Haar;
  - la transforme et la  transforme inverse de l'ondelette de Debauchies:
      4, 6, 8, 10, 12, 14, 16, 18, 24, 32;
  - la transforme et la transforme inverse en ondelette Symmlet:
      6, 8, 10, 12, 14, 16, 18, 20, 22, 24;
  - la transforme et la transforme inverse en ondelette coiflet:
      6, 12, 18, 24, 30, 36, 42, 48, 54, 60;
  - la transforme biorthogonale et la transforme biorthogonale inverse:
      1.3, 1.5, 2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 4.4, 5.5, 6.8

*/

/* Brve introduction aux ondelettes
   =================================

  Les algorithmes d'ondelettes ont deux composants, une fonction d'ondelette et une
  fonction d'chelle (dnomm parfois filtre passe haut et filtre passe bas).

  La fonction d'ondelette utilise deux (ou plus) de mesures pour dterminer un coefficient
  d'ondelette. La fonction d'chelle utilise ces mmes mesures pour crer une version
  comprime de ces mesures.

  Dans la pratique, la premire moiti des donnes est remplace par les valeurs
  dtermines par la fonction d'chelle, la seconde moiti des donnes est remplace
  par les valeurs dtermines par la fonction d'ondelette.

  On peut donc dire que le signal trait est remplac dans la premire moiti par 
  un signal quivalent "en moyenne" mais  frquence d'chantillonnage 2 fois plus 
  faible et que la seconde partie du signal par les coefficients (les dtails) qui
  ajouts  la premire partie du signal permettent de le recomposer identiquement.

  On obtient un traitement en multirsolution en reproduisant ce processus  chaque
  fois sur la premire moiti du signal (le sparant  nouveau en valeurs d'chelles
  et en valeur d'ondelette). Un traitement en multirsolution doit donc se raliser sur
  2^m mesures.
  
*/

/*
  Historique de la bibliothque:
  ******************************

  R0.1: 29 juillet 2004 : premire dition par BDC
  R0.2: 03 aout 2004 : ajout Daubechies 6, 8 et 10 (BDC)
  R0.3: 27 aout 2010 : ajout des ondelettes biorthogonales (BDC)
  R0.4: 31 aout 2010 : rcriture complte du code : introduction de fonctions gnrales (BDC)
  R0.41: 11 fvrier 2011 : ajout de daub 20, 22, symmlet 18, 20, 22, 24 et coiflet 30 (BDC)
  R0.42: 16 novembre 2011 : ajout de daub 32 (BDC)
  R0.43: 05 janvier 2012 : ajout de bio 2.2 et 2.4, coiflet 36, 42, 48, 54, 60 (BDC)

*/

/**************** fonctions ******************/

/* Transforme en ondelettes de Haar
   =================================

Pour calculer la transforme en ondelettes de Haar d'un tableau de n = 2^m donnes:

  - calculer la moyenne de chaque paire de donnes (n/2 moyennes);
  - calculer la diffrence entre chaque donne et sa moyenne respective (n/2 diffrences);
  - placer les moyennes dans la premire moiti du tableau de donnes;
  - placer les diffrences dans la seconde moiti du tableau de donnes;
  - rpter le processus sur la premire moiti des donnes (c'est pourquoi le nombre de
    donnes n doit tre une puissance de 2).

moyenne / diffrence
--------------------

Deux mesures (l,r) peuvent tre exprimes sous la forme d'une moyenne a et d'une
diffrence d de la manire suivante:

  a = (l+r)/2   = la fonction d'chelle
  d = l-a = a-r = (l-r)/2  = la fonction d'ondelette

Le processus tant rversible de la manire suivante:

  l = a+d
  r = a-d

exemple
-------

8 donnes:

    7, 1, 6, 6, 3, -5, 4, 2

  Moyennes:

    (7 +  1) / 2 = 4
    (6 +  6) / 2 = 6
    (3 + -5) / 2 = -1
    (4 +  2) / 2 = 3

  Diffrences:

    (7 - 4) =  ( 4 - 1) = 3
    (6 - 6) =  ( 6 - 6) = 0
    (3 - -1) = (-1 - -5) = 4
    (4 - 3) =  ( 3 - 2) = 1

Le premier traitement donne les valeurs suivantes:

    4, 6, -1, 3, 3, 0, 4, 1

Le traitement se poursuit sur les 4 premires valeurs:

    4, 6, -1, 3

  Moyennes:

    ( 4 + 6) / 2 = 5
    (-1 + 3) / 2 = 1

  Diffrences:

    ( 4 - 5) = (5 - 6) = -1
    (-1 - 1) = (1 - 3) = -2

Le deuxime traitement donne les valeurs suivantes:

    5, 1, -1, -2, 3, 0, 4, 1

Le traitement se poursuit sur les 2 premires valeurs:

    5, 1

  Moyenne:

    (5 + 1) / 2 = 3

  Diffrence:

    (5 - 3) = (3 - 1) = 2

Le troisime traitement donne les valeurs suivantes:

    3, 2, -1, -2, 3, 0, 4, 1

Deux mesures tant le minimum possible, nous ne pouvons pas aller plus en avant dans
ce processus. On remarque que la premire valeur obtenue par la transforme est gale
 la moyenne des donnes initiales:

  (7 + 1 + 6 + 6 + 3 - 5 + 4 + 2) / 8 = 3

Le processus inverse commence sur les deux premires valeurs:

	3, 2

	l = 3 + 2 = 5
	r = 3 - 2 = 1

Le premier traitement donne les valeurs suivantes:

    5, 1, -1, -2, 3, 0, 4, 1

Le traitement se poursuit sur les 4 premires valeurs:

    5, 1, -1, -2

	l1 = 5 + -1 = 4
	r1 = 5 - -1 = 6

	l2 = 1 + -2 = -1
	r2 = 1 - -2 = 3

Le second traitement donne les valeurs suivantes:

    4, 6, -1, 3, 3, 0, 4, 1

Le traitement se poursuit sur les 8 premires valeurs (soit l'ensemble des valeurs):

	4, 6, -1, 3, 3, 0, 4, 1

	l1 = 4 + 3 = 7
	r1 = 4 - 3 = 1

	l2 = 6 + 0 = 6
	r2 = 6 - 0 = 6

	l3 = -1 + 4 = 3
	r3 = -1 - 4 = -5

      l4 = 3 + 1 = 4
	r4 = 3 - 1 = 2

Le troisime et dernier traitement donne les valeurs suivantes:

    7, 1, 6, 6, 3, -5, 4, 2

qui sont bien les valeurs initiales.

*/

int haar(double *valeur, int m) /* transforme de Haar d'un tableau de 2^m valeurs */
{
	long dimension; /* taille des donnes */
	long i, compteur, demi_compteur;
	double *moy, *dif; /* tableau des moyennes et des diffrences */

	dimension = (long) pow(2.0,m); /* taille du tableau des donnes */

	if((moy = (double *) malloc(sizeof(double) * (dimension)))==NULL) {return(0);}
	if((dif = (double *) malloc(sizeof(double) * (dimension)))==NULL) {return(0);}

//	for(compteur=dimension; compteur>1; compteur/=2) // pour une transforme multiple
	compteur=dimension;                      // pour une transforme  un seul niveau
	{
		demi_compteur = compteur/2;

		/* calcul des moyennes et des diffrences de chaque paires */

		for(i=0; i<demi_compteur; i++)
		{
			*(moy+i) = (*(valeur+2*i) + *(valeur+2*i+1)) / 2.0;
			*(dif+i) = *(valeur+2*i) - *(moy+i);
		}

		/* remplacement des valeurs du tableau par les moyennes et diffrences calcules */

		for(i=0; i<demi_compteur; i++)
		{
			*(valeur+i) = *(moy+i);
			*(valeur+i+demi_compteur) = *(dif+i);
		}
	}

	free(moy);
	free(dif);

	return(1);
}

int haar_inv(double *valeur, int m) /* transforme inverse de Haar d'un tableau de 2^m valeurs */
{
	long dimension; /* taille des donnes */
	long i, compteur, demi_compteur;
	double *temp; /* tableau du traitement des moyennes et des diffrences */

	dimension = (long) pow(2.0,m); /* taille du tableau des donnes */

	if((temp = (double *) malloc(sizeof(double) * (dimension)))==NULL) {return(0);}

//	for(compteur = 2; compteur <= dimension; compteur*=2) // pour une inversion multiple
	compteur = dimension;                        //  pour une inversion  un seul niveau
	{
		demi_compteur = compteur/2;

		/* calcul des valeurs de l et r */

		for(i=0; i<demi_compteur; i++)
		{
			*(temp+2*i) = *(valeur+i) + *(valeur+i+demi_compteur);
			*(temp+2*i+1) = *(valeur+i) - *(valeur+i+demi_compteur);
		}

		/* criture des valeurs l et r dans le tableau de valeur */

		for(i=0; i<compteur;i++)
		{
			*(valeur+i) = *(temp+i);
		}

	}

	free(temp);

	return(1);
}

/* Transforme en ondelettes de Haar normalise
   ============================================

  Le calcul est identique  celui de l'ondelette de Haar, mais normalis par
  une multiplication par sqrt(2).

moyenne / diffrence
--------------------

Deux mesures (l,r) peuvent tre exprimes sous la forme d'une moyenne a et d'une
diffrence d de la manire suivante:

  a = (l+r)/sqrt(2)   = la fonction d'chelle
  d = l-a = a-r = (l-r)/sqrt(2)  = la fonction d'ondelette

Le processus tant rversible de la manire suivante:

  l = (a+d)/sqrt(2)
  r = (a-d)/sqrt(2)

*/

int haarn(double *valeur, int m) /* transforme de Haar normalise d'un tableau de 2^m valeurs */
{
	long dimension; /* taille des donnes */
	long i, compteur, demi_compteur;
	double *moy, *dif; /* tableau des moyennes et des diffrences */
	const double sqrt_2 = sqrt(2.0);

	dimension = (long) pow(2.0,m); /* taille du tableau des donnes */

	if((moy = (double *) malloc(sizeof(double) * (dimension)))==NULL) {return(0);}
	if((dif = (double *) malloc(sizeof(double) * (dimension)))==NULL) {return(0);}

//	for(compteur=dimension; compteur>1; compteur/=2) // pour une transforme multiple
	compteur=dimension;                      // pour une transforme  un seul niveau
	{
		demi_compteur = compteur/2;

		/* calcul des moyennes et des diffrences de chaque paires */

		for(i=0; i<demi_compteur; i++)
		{
			*(moy+i) = (*(valeur+2*i) + *(valeur+2*i+1)) / sqrt_2;
			*(dif+i) = (*(valeur+2*i) - *(valeur+2*i+1)) / sqrt_2;
		}

		/* remplacement des valeurs du tableau par les moyennes et diffrences calcules */

		for(i=0; i<demi_compteur; i++)
		{
			*(valeur+i) = *(moy+i);
			*(valeur+i+demi_compteur) = *(dif+i);
		}
	}

	free(moy);
	free(dif);

	return(1);
}

int haarn_inv(double *valeur, int m) /* transforme inverse de Haar normalise d'un tableau de 2^m valeurs */
{
	long dimension; /* taille des donnes */
	long i, compteur, demi_compteur;
	double *temp; /* tableau du traitement des moyennes et des diffrences */
	const double sqrt_2 = sqrt(2.0);

	dimension = (long) pow(2.0,m); /* taille du tableau des donnes */

	if((temp = (double *) malloc(sizeof(double) * (dimension)))==NULL) {return(0);}

//	for(compteur = 2; compteur <= dimension; compteur*=2) // pour une inversion multiple
	compteur = dimension;                        //  pour une inversion  un seul niveau
	{
		demi_compteur = compteur/2;

		/* calcul des valeurs de l et r */

		for(i=0; i<demi_compteur; i++)
		{
			*(temp+2*i) = (*(valeur+i) + *(valeur+i+demi_compteur))/sqrt_2;
			*(temp+2*i+1) = (*(valeur+i) - *(valeur+i+demi_compteur))/sqrt_2;
		}

		/* criture des valeurs l et r dans le tableau de valeur */

		for(i=0; i<compteur;i++)
		{
			*(valeur+i) = *(temp+i);
		}

	}

	free(temp);

	return(1);
}

/* algorithme de Transforme en ondelettes
   =======================================

  L'algorithme de transforme d'ondelette est compos d'une fonction d'ondelette et 
  d'une fonction d'chelle. Les coefficients de la fonction d'chelle sont nots "h" et
  les coefficients de la fonction d'ondelette sont nots "g". L'application de la fonction
  d'chelle donne un coefficient de type "e" et l'application de la fonction d'ondelette
  donne un coefficient de type "o"
  
  La transforme est applique comme une multiplication matricielle du type (pour 4 coefficients):

  | h0 h1 h2 h3  0  0  0  0 | | y1 |   | e1 |                        | e1 |
  | g0 g1 g2 g3  0  0  0  0 | | y2 |   | o1 |                        | e2 |
  |  0  0 h0 h1 h2 h3  0  0 | | y3 |   | e2 |                        | e3 |
  |  0  0 g0 g1 g2 g3  0  0 | | y4 | = | o2 | on permutte pour avoir | e4 |
  |  0  0  0  0 h0 h1 h2 h3 | | y5 |   | e3 |                        | o1 |
  |  0  0  0  0 g0 g1 g2 g3 | | y6 |   | o3 |                        | o2 |
  | h2 h3  0  0  0  0 h0 h1 | | y7 |   | e4 |                        | o3 |
  | g2 g3  0  0  0  0 g0 g1 | | y8 |   | o4 |                        | o4 |

  en rapliquant le traitement 

  | h0 h1 h2 h3 | | e1 |   | e1' |                        | e1' |
  | g0 g1 g2 g3 | | e2 |   | o1' | on permutte pour avoir | e2' |
  | h2 h3 h0 h1 | | e3 | = | e2' |                        | o1' |
  | g2 g3 g0 g1 | | e4 |   | o2' |                        | o2' |

  on obtient la multirsolution

  Dans le cas de la transforme de Daubechies les coefficients de sont calculs (entre autres) 
  pour que la matrice transpose soit la matrice inverse:

  | h0 h1 h2 h3 | | h0 g0 h2 g2 |    | 1 0 0 0 |
  | g0 g1 g2 g3 | | h1 g1 h3 g3 |  = | 0 1 0 0 |
  | h2 h3 h0 h1 | | h2 g2 h0 g0 |    | 0 0 1 0 |
  | g2 g3 g0 g1 | | h3 g3 h1 g1 |    | 0 0 0 1 |

  donc:

  | h0 g0 h2 g2 | | e1' |   | e1 |
  | h1 g1 h3 g3 | | o1' | = | e2 |
  | h2 g2 h0 g0 | | e2' |   | e3 |
  | h3 g3 h1 g1 | | o1' |   | e4 |


  | h0 g0  0  0  0  0 h2 g2 | | e1 |   | y1 |
  | h1 g1  0  0  0  0 h3 g3 | | o1 |   | y2 |
  | h2 g2 h0 g0  0  0  0  0 | | e2 |   | y3 |
  | h3 g3 h1 g1  0  0  0  0 | | o2 | = | y4 |
  |  0  0 h2 g2 h0 g0  0  0 | | e3 |   | y5 |
  |  0  0 h3 g3 h1 g1  0  0 | | o3 |   | y6 |
  |  0  0  0  0 h2 g2 h0 g0 | | e4 |   | y7 |
  |  0  0  0  0 h3 g3 h1 g1 | | o4 |   | y8 |

  qui permet de retrouver le signal de dpart

*/

int transformee(double *valeurs, int m, double *h, double *g, int nc) /* transforme d'un tableau de 2^m valeurs */
{   
/*
Les arguments de la fonction transformee() sont:

  valeurs : tableau des donnes  traiter;
  m  : il y a 2^m donnes dans le tableau valeurs;
  h  : coefficients de la fonction d'chelle;
  g  : coefficients de la fonction d'ondelette;
  nc : nombre de coefficients de l'ondelette;

Ici, la transforme est calcule sur un seul niveau.
Si on veut la calculer sur nn niveaux il faut calculer : c0 = (int) pow(2,m-nn+1);  ( au lieu de c0 = nd; )
Et si on veut directement la transforme multiple il faut utiliser la boucle for prvue pour cela.

*/
    int nd; /* taille des donnes */
    int i, j, k, compteur, demi_compteur;
    int c0, c2, c3;
    double *temp; /* tableau du traitement des moyennes et des diffrences */
  
    nd = (int) pow(2.0,m);    /* taille du tableau des donnes */
    c0 = nd;
    c2 = 2; while(c2<nc) c2=2*c2;
    if(c0<c2) c0=c2;
    c3 = nc-1;
    if((temp = (double *)malloc(nd*sizeof(double)))==NULL) {return(0);}
//  for (compteur = nd; compteur >= c2; compteur/=2)   // pour une transforme multiple
    for (compteur = nd; compteur >= c0; compteur/=2)   // pour une transforme  un seul niveau
    {
        demi_compteur = compteur/2;
        i = 0;
        for (j=0; j<compteur-c3; j+=2)
        {
            temp[i] = 0.0;
            temp[i+demi_compteur] = 0.0;
            for(k=0;k<nc;k++)
            {
                temp[i] += valeurs[j+k]*h[k];
                temp[i+demi_compteur] += valeurs[j+k]*g[k];
            }
            i++;
        } 
        while(i<demi_compteur)
        {
            temp[i] = 0.0;
            temp[i+demi_compteur] = 0.0;
            j=i*2;
            for(k=0;k<nc;k++)
            {
                if(j+k == compteur) {j=-k;}
                temp[i] += valeurs[j+k]*h[k];
                temp[i+demi_compteur] += valeurs[j+k]*g[k];
            }
            i++;
        }
        for (i=0; i<compteur; i++)
        {
            valeurs[i] = temp[i];
        }	      
    }
    free(temp);
    return(1);
}

int inverse(double *valeurs, int m, double *h, double *g, int nc) /* transforme inverse d'un tableau de 2^m valeurs */
{
/*
Les arguments de la fonction inverse() sont:

  valeurs : tableau des donnes  traiter;
  m  : il y a 2^m donnes dans le tableau valeurs;
  h  : coefficients de la fonction d'chelle;
  g  : coefficients de la fonction d'ondelette;
  nc : nombre de coefficients de l'ondelette;

Ici la transforme inverse est calcule sur un seul niveau.
Si on veut la calculer sur nn niveaux il faut calculer : c0 = (int) pow(2,m-nn+1);  ( au lieu de c0 = nd; )
Et si on veut directement la transforme inverse multiple il faut utiliser la boucle for prvue pour cela.

*/
    int nd; /* taille des donnes */
    int i, j, k, compteur, demi_compteur;
    int c0, c2, c3, c4, c5, c6;
    int *ch, *cg; 
    double *temp; /* tableau du traitement des moyennes et des diffrences */
    
    nd = (int) pow(2.0,m); /* taille du tableau des donnes */
    c0 = nd;
    c2 = 2; while(c2<nc) c2=2*c2;
    if(c0<c2) c0=c2;
    c3 = nc-1;
    c4 = nc/2;
    c5 = c4 - 1;
    c6 = nc - 2;
    if((temp = (double *)malloc(nd*sizeof(double)))==NULL) {return(0);}
    if((ch = (int *)malloc(c4*sizeof(int)))==NULL) {return(0);}
    if((cg = (int *)malloc(c4*sizeof(int)))==NULL) {return(0);}
//  for (compteur = c2; compteur <= nd; compteur*=2)  // pour une inversion multiple
    for (compteur = c0; compteur <= nd; compteur*=2)  // pour une inversion  un seul niveau
    {
        demi_compteur = compteur/2;	
        /* initialisation des compteurs de dbut de matrice */
        ch[0] = demi_compteur - c5;
        cg[0] = ch[0] + demi_compteur;
        for(i=1; i<c5; i++)
        {
            ch[i]=ch[i-1] + 1;
            cg[i]=cg[i-1] + 1;
        }
        ch[c5] = 0;
        cg[c5] = demi_compteur;
        for(j=0; j<c6; j+=2)
        {
            temp[j] = 0.0;
            temp[j+1] = 0.0;
            for(k=0;k<c4;k++)
            {
                temp[j]+= valeurs[ch[k]]*h[c6-(k+k)] + valeurs[cg[k]]*g[c6-(k+k)];
                temp[j+1]+= valeurs[ch[k]]*h[c3-(k+k)] + valeurs[cg[k]]*g[c3-(k+k)];
            }
            for(i=0;i<c4;i++)
            {
                ch[i]++;
                if(ch[i] == demi_compteur) {ch[i] = 0;}
                cg[i]=ch[i]+demi_compteur;
            }
        }
        j = c6;
        for (i=0; i<demi_compteur-c5;i++)
        {
            temp[j] = 0;
            temp[j+1] = 0;
            for(k=0;k<c4;k++)
            {
                temp[j]+= valeurs[i+k]*h[c6-(k+k)] + valeurs[i+demi_compteur+k]*g[c6-(k+k)];
                temp[j+1]+= valeurs[i+k]*h[c3-(k+k)] + valeurs[i+demi_compteur+k]*g[c3-(k+k)];
            }
            j+=2;
        }
        for (i=0; i<compteur; i++)
        {
            valeurs[i] = temp[i];
        }
    }
    free(temp);
    free(ch);
    free(cg);
    return(1);
}

int daub4(double *valeur, int m) /* transforme de Daubechies 4 d'un tableau de 2^m valeurs */
{
	double h[4];
	double g[4];
	int i, c1, c2, resultat;

	const double sqrt_3 = sqrt(3.0);
    const double denom = 4 * sqrt(2.0);

	/* calcul des coefficients de Daubechies 4 */
    
    h[0] = (1 + sqrt_3)/denom;
    h[1] = (3 + sqrt_3)/denom;
    h[2] = (3 - sqrt_3)/denom;
    h[3] = (1 - sqrt_3)/denom;
    
    for(i=0;i<4;i++)
	{
		g[i] = pow(-1.0,i) * h[3-i];
	}

	c1 = 4; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 4; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
	
}

int daub4_inv(double *valeur, int m) /* transforme inverse de Daubechies 4 d'un tableau de 2^m valeurs */
{
	double h[4];
	double g[4];

	int i, c1, c2, resultat;
	
	const double sqrt_3 = sqrt(3.0);
    const double denom = 4 * sqrt(2.0);

	/* calcul des coefficients de Daubechies 4 */
    
    h[0] = (1 + sqrt_3)/denom;
    h[1] = (3 + sqrt_3)/denom;
    h[2] = (3 - sqrt_3)/denom;
    h[3] = (1 - sqrt_3)/denom;
    
    for(i=0;i<4;i++)
	{
		g[i] = pow(-1.0,i) * h[3-i];
	}

	c1 = 4; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 4; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
	
}

int daub6(double *valeur, int m) /* transforme de Daubechies 6 d'un tableau de 2^m valeurs */
{
	double h[6];
	double g[6];
	int i, c1, c2, resultat;

	const double sqrt_10 = sqrt(10.0);
	const double sqrt_gr = sqrt(5.0 + 2.0 * sqrt_10);
    const double denom = 16 * sqrt(2.0);

	/* calcul des coefficients de Daubechies 6 */
    
    h[0] = (1.0 + sqrt_10 + sqrt_gr)/denom;
    h[1] = (5.0 + sqrt_10 + 3.0 * sqrt_gr)/denom;
    h[2] = (10.0 - 2.0*sqrt_10 + 2.0 * sqrt_gr)/denom;
    h[3] = (10.0 - 2.0*sqrt_10 - 2.0 * sqrt_gr)/denom;
	h[4] = (5.0 + sqrt_10 - 3.0 * sqrt_gr)/denom;
	h[5] = (1.0 + sqrt_10 - sqrt_gr)/denom;

	for(i=0;i<6;i++)
	{
		g[i] = pow(-1.0,i) * h[5-i];
	}

	c1 = 6; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
	
}


int daub6_inv(double *valeur, int m) /* transforme inverse de Daubechies 6 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double h[6];
	double g[6];

	const double sqrt_10 = sqrt(10.0);
	const double sqrt_gr = sqrt(5.0 + 2.0 * sqrt_10);
    const double denom = 16 * sqrt(2.0);

	/* calcul des coefficients de Daubechies 6 */
    
    h[0] = (1.0 + sqrt_10 + sqrt_gr)/denom;
    h[1] = (5.0 + sqrt_10 + 3.0 * sqrt_gr)/denom;
    h[2] = (10.0 - 2.0*sqrt_10 + 2.0 * sqrt_gr)/denom;
    h[3] = (10.0 - 2.0*sqrt_10 - 2.0 * sqrt_gr)/denom;
	h[4] = (5.0 + sqrt_10 - 3.0 * sqrt_gr)/denom;
	h[5] = (1.0 + sqrt_10 - sqrt_gr)/denom;

	for(i=0;i<6;i++)
	{
		g[i] = pow(-1.0,i) * h[5-i];
	}

	c1 = 6; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);		
}


int daub8(double *valeur, int m) /* transforme de Daubechies 8 d'un tableau de 2^m valeurs */
{
	double h[8] = {2.303778133088965008632911830440708500016152482483092977910968e-01,
	7.148465705529156470899219552739926037076084010993081758450110e-01,
	6.308807679298589078817163383006152202032229226771951174057473e-01,
	-2.798376941685985421141374718007538541198732022449175284003358e-02,
	-1.870348117190930840795706727890814195845441743745800912057770e-01,
	3.084138183556076362721936253495905017031482172003403341821219e-02,
	3.288301166688519973540751354924438866454194113754971259727278e-02,
	-1.059740178506903210488320852402722918109996490637641983484974e-02};
	double g[8] ;
	int i, c1, c2, resultat;

	for(i=0;i<8;i++)
	{
		g[i] = pow(-1.0,i) * h[7-i];
	}

	c1 = 8; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub8_inv(double *valeur, int m) /* transforme inverse de Daubechies 8 d'un tableau de 2^m valeurs */
{
	double h[8] = {2.303778133088965008632911830440708500016152482483092977910968e-01,
	7.148465705529156470899219552739926037076084010993081758450110e-01,
	6.308807679298589078817163383006152202032229226771951174057473e-01,
	-2.798376941685985421141374718007538541198732022449175284003358e-02,
	-1.870348117190930840795706727890814195845441743745800912057770e-01,
	3.084138183556076362721936253495905017031482172003403341821219e-02,
	3.288301166688519973540751354924438866454194113754971259727278e-02,
	-1.059740178506903210488320852402722918109996490637641983484974e-02};
	double g[8] ;
	int i, c1, c2, resultat;

	for(i=0;i<8;i++)
	{
		g[i] = pow(-1.0,i) * h[7-i];
	}

	c1 = 8; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub10(double *valeur, int m) /* transforme de Daubechies 10 d'un tableau de 2^m valeurs */
{
	double h[10] = {1.601023979741929144807237480204207336505441246250578327725699e-01,
	6.038292697971896705401193065250621075074221631016986987969283e-01,
	7.243085284377729277280712441022186407687562182320073725767335e-01,
	1.384281459013207315053971463390246973141057911739561022694652e-01,
	-2.422948870663820318625713794746163619914908080626185983913726e-01,
	-3.224486958463837464847975506213492831356498416379847225434268e-02,
	7.757149384004571352313048938860181980623099452012527983210146e-02,
	-6.241490212798274274190519112920192970763557165687607323417435e-03,
	-1.258075199908199946850973993177579294920459162609785020169232e-02,
	3.335725285473771277998183415817355747636524742305315099706428e-03};
	double g[10] ;
	int i, c1, c2, resultat;

	for(i=0;i<10;i++)
	{
		g[i] = pow(-1.0,i) * h[9-i];
	}

	c1 = 10; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub10_inv(double *valeur, int m) /* transforme inverse de Daubechies 10 d'un tableau de 2^m valeurs */
{
	double h[10] = {1.601023979741929144807237480204207336505441246250578327725699e-01,
	6.038292697971896705401193065250621075074221631016986987969283e-01,
	7.243085284377729277280712441022186407687562182320073725767335e-01,
	1.384281459013207315053971463390246973141057911739561022694652e-01,
	-2.422948870663820318625713794746163619914908080626185983913726e-01,
	-3.224486958463837464847975506213492831356498416379847225434268e-02,
	7.757149384004571352313048938860181980623099452012527983210146e-02,
	-6.241490212798274274190519112920192970763557165687607323417435e-03,
	-1.258075199908199946850973993177579294920459162609785020169232e-02,
	3.335725285473771277998183415817355747636524742305315099706428e-03};
	double g[10] ;
	int i, c1, c2, resultat;

	for(i=0;i<10;i++)
	{
		g[i] = pow(-1.0,i) * h[9-i];
	}

	c1 = 10; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub12(double *valeur, int m) /* transforme de Daubechies 12 d'un tableau de 2^m valeurs */
{
	double h[12] = {0.11154074335010946362132391724092343904253959198442,
	0.49462389039845308567720417687785558863778638289627,
	0.75113390802109535067893449843973168558025478333826,
	0.31525035170919762908598965481092639664951992351729,
	-0.2262646939654398200763145006609034656705401539729,
	-0.12976686756726193556228960587658546084523374922358,
	0.097501605587323049102343552538125342339830747495255,
	0.027522865530305728625540839504193213657387587830435,
	-0.031582039317486029565079080699848669057479532373148,
	0.0005538422011614961392519183980465012206110262773865,
	0.0047772575109455106396359752468207070502305012165814,
	-0.0010773010853084795648526216095872000352352336093344};
	double g[12] ;
	int i, c1, c2, resultat;

	for(i=0;i<12;i++)
	{
		g[i] = pow(-1.0,i) * h[11-i];
	}

	c1 = 12; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
	
}

int daub12_inv(double *valeur, int m) /* transforme inverse de Daubechies 12 d'un tableau de 2^m valeurs */
{
	double h[12] = {0.11154074335010946362132391724092343904253959198442,
	0.49462389039845308567720417687785558863778638289627,
	0.75113390802109535067893449843973168558025478333826,
	0.31525035170919762908598965481092639664951992351729,
	-0.2262646939654398200763145006609034656705401539729,
	-0.12976686756726193556228960587658546084523374922358,
	0.097501605587323049102343552538125342339830747495255,
	0.027522865530305728625540839504193213657387587830435,
	-0.031582039317486029565079080699848669057479532373148,
	0.0005538422011614961392519183980465012206110262773865,
	0.0047772575109455106396359752468207070502305012165814,
	-0.0010773010853084795648526216095872000352352336093344};
	double g[12] ;
	int i, c1, c2, resultat;

	for(i=0;i<12;i++)
	{
		g[i] = pow(-1.0,i) * h[11-i];
	}

	c1 = 12; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub14(double *valeur, int m) /* transforme de Daubechies 14 d'un tableau de 2^m valeurs */
{
	double h[14] = {0.077852054085009179019963521957893748379183052927956,
	0.39653931948191730653900039093684285635871511493333,
	0.72913209084623511991694307033928205171796606119014,
	0.46978228740519312247159116097445173868179130567874,
	-0.14390600392856497540506836221304600179527357054991,
	-0.22403618499387498263814042023325096447578308967732,
	0.071309219266830264750876570501129048227113274514123,
	0.080612609151083071912922480359381905858238209656295,
	-0.038029936935014413579592061601858035854461969384679,
	-0.016574541630666880654107674891702654792045043948207,
	0.012550998556099840612989886034187779572894740460487,
	0.00042957797292136652113212912281973222282353503969424,
	-0.0018016407040474909152682629127395509625856514696411,
	0.000353713799974520248446295836306425431095906005952};
	double g[14] ;
	int i, c1, c2, resultat;

	for(i=0;i<14;i++)
	{
		g[i] = pow(-1.0,i) * h[13-i];
	}

	c1 = 14; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub14_inv(double *valeur, int m) /* transforme inverse de Daubechies 14 d'un tableau de 2^m valeurs */
{
	double h[14] = {0.077852054085009179019963521957893748379183052927956,
	0.39653931948191730653900039093684285635871511493333,
	0.72913209084623511991694307033928205171796606119014,
	0.46978228740519312247159116097445173868179130567874,
	-0.14390600392856497540506836221304600179527357054991,
	-0.22403618499387498263814042023325096447578308967732,
	0.071309219266830264750876570501129048227113274514123,
	0.080612609151083071912922480359381905858238209656295,
	-0.038029936935014413579592061601858035854461969384679,
	-0.016574541630666880654107674891702654792045043948207,
	0.012550998556099840612989886034187779572894740460487,
	0.00042957797292136652113212912281973222282353503969424,
	-0.0018016407040474909152682629127395509625856514696411,
	0.000353713799974520248446295836306425431095906005952};
	double g[14] ;
	int i, c1, c2, resultat;

	for(i=0;i<14;i++)
	{
		g[i] = pow(-1.0,i) * h[13-i];
	}

	c1 = 14; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub16(double *valeur, int m) /* transforme de Daubechies 16 d'un tableau de 2^m valeurs */
{
	double h[16] = {0.05441584224310400995500940520299935503599554294733,
	0.31287159091429997065916237550571772194973197403702,
	0.6756307362972898068078007670471831499869115906336,
	0.58535468365420671277126552004509819443032666780534,
	-0.015829105256349305667380547876466304157744711545026,
	-0.28401554296154692651620313237416473246843501248714,
	0.00047248457391328277036059000982589498619480112888299,
	0.12874742662047845885702928750970838430226015755565,
	-0.017369301001807546169616148868095983114130865294882,
	-0.044088253930794751506763723238963501897518391901109,
	0.013981027917398281648722930572633451442395595329344,
	0.008746094047405776716382743246475640180402147081141,
	-0.004870352993451574310422181557109824016634978512157,
	-0.00039174037337694704629808035732377626752293500738908,
	0.00067544940645056936636954757387929912184896300135583,
	-0.00011747678412476953373062823169889094440866939503115};
	double g[16] ;
	int i, c1, c2, resultat;

	for(i=0;i<16;i++)
	{
		g[i] = pow(-1.0,i) * h[15-i];
	}

	c1 = 16; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub16_inv(double *valeur, int m) /* transforme inverse de Daubechies 16 d'un tableau de 2^m valeurs */
{
	double h[16] = {0.05441584224310400995500940520299935503599554294733,
	0.31287159091429997065916237550571772194973197403702,
	0.6756307362972898068078007670471831499869115906336,
	0.58535468365420671277126552004509819443032666780534,
	-0.015829105256349305667380547876466304157744711545026,
	-0.28401554296154692651620313237416473246843501248714,
	0.00047248457391328277036059000982589498619480112888299,
	0.12874742662047845885702928750970838430226015755565,
	-0.017369301001807546169616148868095983114130865294882,
	-0.044088253930794751506763723238963501897518391901109,
	0.013981027917398281648722930572633451442395595329344,
	0.008746094047405776716382743246475640180402147081141,
	-0.004870352993451574310422181557109824016634978512157,
	-0.00039174037337694704629808035732377626752293500738908,
	0.00067544940645056936636954757387929912184896300135583,
	-0.00011747678412476953373062823169889094440866939503115};
	double g[16] ;
	int i, c1, c2, resultat;

	for(i=0;i<16;i++)
	{
		g[i] = pow(-1.0,i) * h[15-i];
	}

	c1 = 16; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub18(double *valeur, int m) /* transforme de Daubechies 18 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[18];
	double h[18] = {
 3.807794736387834658869765887955118448771714496278417476647192e-02,
 2.438346746125903537320415816492844155263611085609231361429088e-01,
 6.048231236901111119030768674342361708959562711896117565333713e-01,
 6.572880780513005380782126390451732140305858669245918854436034e-01,
 1.331973858250075761909549458997955536921780768433661136154346e-01,
-2.932737832791749088064031952421987310438961628589906825725112e-01,
-9.684078322297646051350813353769660224825458104599099679471267e-02,
 1.485407493381063801350727175060423024791258577280603060771649e-01,
 3.072568147933337921231740072037882714105805024670744781503060e-02,
-6.763282906132997367564227482971901592578790871353739900748331e-02,
 2.509471148314519575871897499885543315176271993709633321834164e-04,
 2.236166212367909720537378270269095241855646688308853754721816e-02,
-4.723204757751397277925707848242465405729514912627938018758526e-03,
-4.281503682463429834496795002314531876481181811463288374860455e-03,
 1.847646883056226476619129491125677051121081359600318160732515e-03,
 2.303857635231959672052163928245421692940662052463711972260006e-04,
-2.519631889427101369749886842878606607282181543478028214134265e-04,
 3.934732031627159948068988306589150707782477055517013507359938e-05};

    
	for(i=0;i<18;i++)
	{
		g[i] = pow(-1.0,i) * h[17-i];
	}


	c1 = 18; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub18_inv(double *valeur, int m) /* transforme inverse de Daubechies 18 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[18];
	double h[18] = {
 3.807794736387834658869765887955118448771714496278417476647192e-02,
 2.438346746125903537320415816492844155263611085609231361429088e-01,
 6.048231236901111119030768674342361708959562711896117565333713e-01,
 6.572880780513005380782126390451732140305858669245918854436034e-01,
 1.331973858250075761909549458997955536921780768433661136154346e-01,
-2.932737832791749088064031952421987310438961628589906825725112e-01,
-9.684078322297646051350813353769660224825458104599099679471267e-02,
 1.485407493381063801350727175060423024791258577280603060771649e-01,
 3.072568147933337921231740072037882714105805024670744781503060e-02,
-6.763282906132997367564227482971901592578790871353739900748331e-02,
 2.509471148314519575871897499885543315176271993709633321834164e-04,
 2.236166212367909720537378270269095241855646688308853754721816e-02,
-4.723204757751397277925707848242465405729514912627938018758526e-03,
-4.281503682463429834496795002314531876481181811463288374860455e-03,
 1.847646883056226476619129491125677051121081359600318160732515e-03,
 2.303857635231959672052163928245421692940662052463711972260006e-04,
-2.519631889427101369749886842878606607282181543478028214134265e-04,
 3.934732031627159948068988306589150707782477055517013507359938e-05};

    
	for(i=0;i<18;i++)
	{
		g[i] = pow(-1.0,i) * h[17-i];
	}

	c1 = 18; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int daub20(double *valeur, int m) /* transforme de Daubechies 20 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[20];
	double h[20] = {
2.667005790055555358661744877130858277192498290851289932779975e-02,
 1.881768000776914890208929736790939942702546758640393484348595e-01,
 5.272011889317255864817448279595081924981402680840223445318549e-01,
 6.884590394536035657418717825492358539771364042407339537279681e-01,
 2.811723436605774607487269984455892876243888859026150413831543e-01,
-2.498464243273153794161018979207791000564669737132073715013121e-01,
-1.959462743773770435042992543190981318766776476382778474396781e-01,
 1.273693403357932600826772332014009770786177480422245995563097e-01,
 9.305736460357235116035228983545273226942917998946925868063974e-02,
-7.139414716639708714533609307605064767292611983702150917523756e-02,
-2.945753682187581285828323760141839199388200516064948779769654e-02,
 3.321267405934100173976365318215912897978337413267096043323351e-02,
 3.606553566956169655423291417133403299517350518618994762730612e-03,
-1.073317548333057504431811410651364448111548781143923213370333e-02,
 1.395351747052901165789318447957707567660542855688552426721117e-03,
 1.992405295185056117158742242640643211762555365514105280067936e-03,
-6.858566949597116265613709819265714196625043336786920516211903e-04,
-1.164668551292854509514809710258991891527461854347597362819235e-04,
 9.358867032006959133405013034222854399688456215297276443521873e-05,
-1.326420289452124481243667531226683305749240960605829756400674e-05};

    
	for(i=0;i<20;i++)
	{
		g[i] = pow(-1.0,i) * h[19-i];
	}


	c1 = 20; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub20_inv(double *valeur, int m) /* transforme inverse de Daubechies 20 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[20];
	double h[20] = {
2.667005790055555358661744877130858277192498290851289932779975e-02,
 1.881768000776914890208929736790939942702546758640393484348595e-01,
 5.272011889317255864817448279595081924981402680840223445318549e-01,
 6.884590394536035657418717825492358539771364042407339537279681e-01,
 2.811723436605774607487269984455892876243888859026150413831543e-01,
-2.498464243273153794161018979207791000564669737132073715013121e-01,
-1.959462743773770435042992543190981318766776476382778474396781e-01,
 1.273693403357932600826772332014009770786177480422245995563097e-01,
 9.305736460357235116035228983545273226942917998946925868063974e-02,
-7.139414716639708714533609307605064767292611983702150917523756e-02,
-2.945753682187581285828323760141839199388200516064948779769654e-02,
 3.321267405934100173976365318215912897978337413267096043323351e-02,
 3.606553566956169655423291417133403299517350518618994762730612e-03,
-1.073317548333057504431811410651364448111548781143923213370333e-02,
 1.395351747052901165789318447957707567660542855688552426721117e-03,
 1.992405295185056117158742242640643211762555365514105280067936e-03,
-6.858566949597116265613709819265714196625043336786920516211903e-04,
-1.164668551292854509514809710258991891527461854347597362819235e-04,
 9.358867032006959133405013034222854399688456215297276443521873e-05,
-1.326420289452124481243667531226683305749240960605829756400674e-05};

    
	for(i=0;i<20;i++)
	{
		g[i] = pow(-1.0,i) * h[19-i];
	}


	c1 = 20; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int daub22(double *valeur, int m) /* transforme de Daubechies 22 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[22];
	double h[22] = {
1.869429776147108402543572939561975728967774455921958543286692e-02,
 1.440670211506245127951915849361001143023718967556239604318852e-01,
 4.498997643560453347688940373853603677806895378648933474599655e-01,
 6.856867749162005111209386316963097935940204964567703495051589e-01,
 4.119643689479074629259396485710667307430400410187845315697242e-01,
-1.622752450274903622405827269985511540744264324212130209649667e-01,
-2.742308468179469612021009452835266628648089521775178221905778e-01,
 6.604358819668319190061457888126302656753142168940791541113457e-02,
 1.498120124663784964066562617044193298588272420267484653796909e-01,
-4.647995511668418727161722589023744577223260966848260747450320e-02,
-6.643878569502520527899215536971203191819566896079739622858574e-02,
 3.133509021904607603094798408303144536358105680880031964936445e-02,
 2.084090436018106302294811255656491015157761832734715691126692e-02,
-1.536482090620159942619811609958822744014326495773000120205848e-02,
-3.340858873014445606090808617982406101930658359499190845656731e-03,
 4.928417656059041123170739741708273690285547729915802418397458e-03,
-3.085928588151431651754590726278953307180216605078488581921562e-04,
-8.930232506662646133900824622648653989879519878620728793133358e-04,
 2.491525235528234988712216872666801088221199302855425381971392e-04,
 5.443907469936847167357856879576832191936678525600793978043688e-05,
-3.463498418698499554128085159974043214506488048233458035943601e-05,
 4.494274277236510095415648282310130916410497987383753460571741e-06};

    
	for(i=0;i<22;i++)
	{
		g[i] = pow(-1.0,i) * h[21-i];
	}


	c1 = 22; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int daub22_inv(double *valeur, int m) /* transforme inverse de Daubechies 22 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[22];
	double h[22] = {
1.869429776147108402543572939561975728967774455921958543286692e-02,
 1.440670211506245127951915849361001143023718967556239604318852e-01,
 4.498997643560453347688940373853603677806895378648933474599655e-01,
 6.856867749162005111209386316963097935940204964567703495051589e-01,
 4.119643689479074629259396485710667307430400410187845315697242e-01,
-1.622752450274903622405827269985511540744264324212130209649667e-01,
-2.742308468179469612021009452835266628648089521775178221905778e-01,
 6.604358819668319190061457888126302656753142168940791541113457e-02,
 1.498120124663784964066562617044193298588272420267484653796909e-01,
-4.647995511668418727161722589023744577223260966848260747450320e-02,
-6.643878569502520527899215536971203191819566896079739622858574e-02,
 3.133509021904607603094798408303144536358105680880031964936445e-02,
 2.084090436018106302294811255656491015157761832734715691126692e-02,
-1.536482090620159942619811609958822744014326495773000120205848e-02,
-3.340858873014445606090808617982406101930658359499190845656731e-03,
 4.928417656059041123170739741708273690285547729915802418397458e-03,
-3.085928588151431651754590726278953307180216605078488581921562e-04,
-8.930232506662646133900824622648653989879519878620728793133358e-04,
 2.491525235528234988712216872666801088221199302855425381971392e-04,
 5.443907469936847167357856879576832191936678525600793978043688e-05,
-3.463498418698499554128085159974043214506488048233458035943601e-05,
 4.494274277236510095415648282310130916410497987383753460571741e-06};

    
	for(i=0;i<22;i++)
	{
		g[i] = pow(-1.0,i) * h[21-i];
	}


	c1 = 22; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}


int daub24(double *valeur, int m) /* transforme de Daubechies 24 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[24];
	double h[24] = {
 1.311225795722951750674609088893328065665510641931325007748280e-02,
 1.095662728211851546057045050248905426075680503066774046383657e-01,
 3.773551352142126570928212604879206149010941706057526334705839e-01,
 6.571987225793070893027611286641169834250203289988412141394281e-01,
 5.158864784278156087560326480543032700677693087036090056127647e-01,
-4.476388565377462666762747311540166529284543631505924139071704e-02,
-3.161784537527855368648029353478031098508839032547364389574203e-01,
-2.377925725606972768399754609133225784553366558331741152482612e-02,
 1.824786059275796798540436116189241710294771448096302698329011e-01,
 5.359569674352150328276276729768332288862665184192705821636342e-03,
-9.643212009650708202650320534322484127430880143045220514346402e-02,
 1.084913025582218438089010237748152188661630567603334659322512e-02,
 4.154627749508444073927094681906574864513532221388374861287078e-02,
-1.221864906974828071998798266471567712982466093116558175344811e-02,
-1.284082519830068329466034471894728496206109832314097633275225e-02,
 6.711499008795509177767027068215672450648112185856456740379455e-03,
 2.248607240995237599950865211267234018343199786146177099262010e-03,
-2.179503618627760471598903379584171187840075291860571264980942e-03,
 6.545128212509595566500430399327110729111770568897356630714552e-06,
 3.886530628209314435897288837795981791917488573420177523436096e-04,
-8.850410920820432420821645961553726598738322151471932808015443e-05,
-2.424154575703078402978915320531719580423778362664282239377532e-05,
 1.277695221937976658714046362616620887375960941439428756055353e-05,
-1.529071758068510902712239164522901223197615439660340672602696e-06};


    
	for(i=0;i<24;i++)
	{
		g[i] = pow(-1.0,i) * h[23-i]; /* 23 = 24 - 1 */
	}

	c1 = 24; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
	
}


int daub24_inv(double *valeur, int m) /* transforme inverse de Daubechies 24 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[24];
	double h[24] = {
1.311225795722951750674609088893328065665510641931325007748280e-02,
 1.095662728211851546057045050248905426075680503066774046383657e-01,
 3.773551352142126570928212604879206149010941706057526334705839e-01,
 6.571987225793070893027611286641169834250203289988412141394281e-01,
 5.158864784278156087560326480543032700677693087036090056127647e-01,
-4.476388565377462666762747311540166529284543631505924139071704e-02,
-3.161784537527855368648029353478031098508839032547364389574203e-01,
-2.377925725606972768399754609133225784553366558331741152482612e-02,
 1.824786059275796798540436116189241710294771448096302698329011e-01,
 5.359569674352150328276276729768332288862665184192705821636342e-03,
-9.643212009650708202650320534322484127430880143045220514346402e-02,
 1.084913025582218438089010237748152188661630567603334659322512e-02,
 4.154627749508444073927094681906574864513532221388374861287078e-02,
-1.221864906974828071998798266471567712982466093116558175344811e-02,
-1.284082519830068329466034471894728496206109832314097633275225e-02,
 6.711499008795509177767027068215672450648112185856456740379455e-03,
 2.248607240995237599950865211267234018343199786146177099262010e-03,
-2.179503618627760471598903379584171187840075291860571264980942e-03,
 6.545128212509595566500430399327110729111770568897356630714552e-06,
 3.886530628209314435897288837795981791917488573420177523436096e-04,
-8.850410920820432420821645961553726598738322151471932808015443e-05,
-2.424154575703078402978915320531719580423778362664282239377532e-05,
 1.277695221937976658714046362616620887375960941439428756055353e-05,
-1.529071758068510902712239164522901223197615439660340672602696e-06};

    
	for(i=0;i<24;i++)
	{
		g[i] = pow(-1.0,i) * h[23-i]; /* 23 = 24 - 1 */
	}

	c1 = 24; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	

}


int daub32(double *valeur, int m) /* transforme de Daubechies 32 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[32];
	double h[32] = {
 3.189220925347738029769547564645958687067086750131428767875878e-03,
 3.490771432367334641030147224023020009218241430503984146140054e-02,
 1.650642834888531178991252730561134811584835002342723240213592e-01,
 4.303127228460038137403925424357684620633970478036986773924646e-01,
 6.373563320837888986319852412996030536498595940814198125967751e-01,
 4.402902568863569000390869163571679288527803035135272578789884e-01,
-8.975108940248964285718718077442597430659247445582660149624718e-02,
-3.270633105279177046462905675689119641757228918228812428141723e-01,
-2.791820813302827668264519595026873204339971219174736041535479e-02,
 2.111906939471042887209680163268837900928491426167679439251042e-01,
 2.734026375271604136485245757201617965429027819507130220231500e-02,
-1.323883055638103904500474147756493375092287817706027978798549e-01,
-6.239722752474871765674503394120025865444656311678760990761458e-03,
 7.592423604427631582148498743941422461530405946100943351940313e-02,
-7.588974368857737638494890864636995796586975144990925400097160e-03,
-3.688839769173014233352666320894554314718748429706730831064068e-02,
 1.029765964095596941165000580076616900528856265803662208854147e-02,
 1.399376885982873102950451873670329726409840291727868988490100e-02,
-6.990014563413916670284249536517288338057856199646469078115759e-03,
-3.644279621498389932169000540933629387055333973353108668841215e-03,
 3.128023381206268831661202559854678767821471906193608117450360e-03,
 4.078969808497128362417470323406095782431952972310546715071397e-04,
-9.410217493595675889266453953635875407754747216734480509250273e-04,
 1.142415200387223926440228099555662945839684344936472652877091e-04,
 1.747872452253381803801758637660746874986024728615399897971953e-04,
-6.103596621410935835162369150522212811957259981965919143961722e-05,
-1.394566898820889345199078311998401982325273569198675335408707e-05,
 1.133660866127625858758848762886536997519471068203753661757843e-05,
-1.043571342311606501525454737262615404887478930635676471546032e-06,
-7.363656785451205512099695719725563646585445545841663327433569e-07,
 2.308784086857545866405412732942006121306306735866655525372544e-07,
-2.109339630100743097000572623603489906836297584591605307745349e-08};

    
	for(i=0;i<32;i++)
	{
		g[i] = pow(-1.0,i) * h[31-i]; /* 31 = 32 - 1 */
	}

	c1 = 32; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
	
}

int daub32_inv(double *valeur, int m) /* transforme inverse de Daubechies 32 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[32];
	double h[32] = {
 3.189220925347738029769547564645958687067086750131428767875878e-03,
 3.490771432367334641030147224023020009218241430503984146140054e-02,
 1.650642834888531178991252730561134811584835002342723240213592e-01,
 4.303127228460038137403925424357684620633970478036986773924646e-01,
 6.373563320837888986319852412996030536498595940814198125967751e-01,
 4.402902568863569000390869163571679288527803035135272578789884e-01,
-8.975108940248964285718718077442597430659247445582660149624718e-02,
-3.270633105279177046462905675689119641757228918228812428141723e-01,
-2.791820813302827668264519595026873204339971219174736041535479e-02,
 2.111906939471042887209680163268837900928491426167679439251042e-01,
 2.734026375271604136485245757201617965429027819507130220231500e-02,
-1.323883055638103904500474147756493375092287817706027978798549e-01,
-6.239722752474871765674503394120025865444656311678760990761458e-03,
 7.592423604427631582148498743941422461530405946100943351940313e-02,
-7.588974368857737638494890864636995796586975144990925400097160e-03,
-3.688839769173014233352666320894554314718748429706730831064068e-02,
 1.029765964095596941165000580076616900528856265803662208854147e-02,
 1.399376885982873102950451873670329726409840291727868988490100e-02,
-6.990014563413916670284249536517288338057856199646469078115759e-03,
-3.644279621498389932169000540933629387055333973353108668841215e-03,
 3.128023381206268831661202559854678767821471906193608117450360e-03,
 4.078969808497128362417470323406095782431952972310546715071397e-04,
-9.410217493595675889266453953635875407754747216734480509250273e-04,
 1.142415200387223926440228099555662945839684344936472652877091e-04,
 1.747872452253381803801758637660746874986024728615399897971953e-04,
-6.103596621410935835162369150522212811957259981965919143961722e-05,
-1.394566898820889345199078311998401982325273569198675335408707e-05,
 1.133660866127625858758848762886536997519471068203753661757843e-05,
-1.043571342311606501525454737262615404887478930635676471546032e-06,
-7.363656785451205512099695719725563646585445545841663327433569e-07,
 2.308784086857545866405412732942006121306306735866655525372544e-07,
-2.109339630100743097000572623603489906836297584591605307745349e-08};

    
	for(i=0;i<32;i++)
	{
		g[i] = pow(-1.0,i) * h[31-i]; /* 31 = 32 - 1 */
	}

	c1 = 32; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	

}




/* Transforme en ondelettes Symmlet
   =================================

  Le principe de l'algorithme de transforme en ondelette Symmlet est identique
   celui implment pour les ondelettes de Daubechies. Seul les coefficients des
  fonction d'chelle et d'ondelette sont modifis.

*/

int symmlet6(double *valeur, int m) /* transforme de Symmlet 6 d'un tableau de 2^m valeurs */
{
	double h[6] = {0.33267055295096, 0.80689150931334, 0.45987750211933, -0.13501102001039,
					-0.08544127388224, 0.03522629188210};
	double g[6] ;
	int i, c1, c2, resultat;

	for(i=0;i<6;i++)
	{
		g[i] = pow(-1.0,i) * h[5-i];
	}

	c1 = 6; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);	
}

int symmlet6_inv(double *valeur, int m) /* transforme inverse de Symmlet 6 d'un tableau de 2^m valeurs */
{
	double h[6] = {0.33267055295096, 0.80689150931334, 0.45987750211933, -0.13501102001039,
					-0.08544127388224, 0.03522629188210};
	double g[6] ;
	int i, c1, c2, resultat;

	for(i=0;i<6;i++)
	{
		g[i] = pow(-1.0,i) * h[5-i];
	}

	c1 = 6; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet8(double *valeur, int m) /* transforme de Symmlet 8 d'un tableau de 2^m valeurs */
{
	double h[8] = {0.0322231006040782, -0.0126039672622638,	-0.0992195435769564, 0.297857795605605,
					0.803738751805386, 0.497618667632563, -0.0296355276459604, -0.0757657147893567};
	double g[8] ;
	int i, c1, c2, resultat;

	for(i=0;i<8;i++)
	{
		g[i] = pow(-1.0,i) * h[7-i];
	}

	c1 = 8; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
	
}

int symmlet8_inv(double *valeur, int m) /* transforme inverse de Symmlet 8 d'un tableau de 2^m valeurs */
{
	
	double h[8] = {0.0322231006040782, -0.0126039672622638,	-0.0992195435769564, 0.297857795605605,
					0.803738751805386, 0.497618667632563, -0.0296355276459604, -0.0757657147893567};
	double g[8] ;
	int i, c1, c2, resultat;

	for(i=0;i<8;i++)
	{
		g[i] = pow(-1.0,i) * h[7-i];
	}

	c1 = 8; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet10(double *valeur, int m) /* transforme de Symmlet 10 d'un tableau de 2^m valeurs */
{
	double h[10] = {0.01953888273529, -0.02110183402476, -0.17532808990845, 0.01660210576452, 0.63397896345821,
					0.72340769040242, 0.19939753397739, -0.03913424930238, 0.02951949092577, 0.02733306834508};
	double g[10] ;
	int i, c1, c2, resultat;

	for(i=0;i<10;i++)
	{
		g[i] = pow(-1.0,i) * h[9-i];
	}

	c1 = 10; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet10_inv(double *valeur, int m) /* transforme inverse de Symmlet 10 d'un tableau de 2^m valeurs */
{
	double h[10] = {0.01953888273529, -0.02110183402476, -0.17532808990845, 0.01660210576452, 0.63397896345821,
					0.72340769040242, 0.19939753397739, -0.03913424930238, 0.02951949092577, 0.02733306834508};
	double g[10] ;
	int i, c1, c2, resultat;

	for(i=0;i<10;i++)
	{
		g[i] = pow(-1.0,i) * h[9-i];
	}

	c1 = 10; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet12(double *valeur, int m) /* transforme de symmlet 12 d'un tableau de 2^m valeurs */
{
	double h[12] = {-0.00780070832503, 0.00176771186424, 0.04472490177067, -0.02106029251230, -0.07263752278646,
					0.33792942172762, 0.78764114103019, 0.49105594192675, -0.04831174258563, -0.11799011114819,
					0.00349071208422, 0.01540410932703};
	double g[12] ;
	int i, c1, c2, resultat;

	for(i=0;i<12;i++)
	{
		g[i] = pow(-1.0,i) * h[11-i];
	}

	c1 = 12; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet12_inv(double *valeur, int m) /* transforme inverse de symmlet 12 d'un tableau de 2^m valeurs */
{
	double h[12] = {-0.00780070832503, 0.00176771186424, 0.04472490177067, -0.02106029251230, -0.07263752278646,
					0.33792942172762, 0.78764114103019, 0.49105594192675, -0.04831174258563, -0.11799011114819,
					0.00349071208422, 0.01540410932703};
	double g[12] ;
	int i, c1, c2, resultat;

	for(i=0;i<12;i++)
	{
		g[i] = pow(-1.0,i) * h[11-i];
	}

	c1 = 12; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet14(double *valeur, int m) /* transforme de symmlet 14 d'un tableau de 2^m valeurs */
{
	double h[14] = {0.01026817670851, 0.00401024487153, -0.10780823770382, -0.14004724044296, 0.28862963175151,
					0.76776431700316, 0.53610191709176, 0.01744125508686, -0.04955283493713, 0.06789269350137,
					0.03051551316596, -0.01263630340325, -0.00104738488868, 0.00268181456826};
	double g[14] ;
	int i, c1, c2, resultat;

	for(i=0;i<14;i++)
	{
		g[i] = pow(-1.0,i) * h[13-i];
	}

	c1 = 14; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);	    
}

int symmlet14_inv(double *valeur, int m) /* transforme inverse de Symmlet 14 d'un tableau de 2^m valeurs */
{
	double h[14] = {0.01026817670851, 0.00401024487153, -0.10780823770382, -0.14004724044296, 0.28862963175151,
					0.76776431700316, 0.53610191709176, 0.01744125508686, -0.04955283493713, 0.06789269350137,
					0.03051551316596, -0.01263630340325, -0.00104738488868, 0.00268181456826};
	double g[14] ;
	int i, c1, c2, resultat;

	for(i=0;i<14;i++)
	{
		g[i] = pow(-1.0,i) * h[13-i];
	}

	c1 = 14; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet16(double *valeur, int m) /* transforme de Symmlet 16 d'un tableau de 2^m valeurs */
{
	double h[16] = {0.00188995033276, -0.00030292051472, -0.01495225833705, 0.00380875201389, 0.04913717967361,
					-0.02721902991706, -0.05194583810771, 0.36444189483534, 0.77718575170053, 0.48135965125838,
					-0.06127335906766, -0.14329423835081, 0.00760748732492, 0.03169508781149, -0.00054213233179,
					-0.00338241595101};
	double g[16] ;
	int i, c1, c2, resultat;

	for(i=0;i<16;i++)
	{
		g[i] = pow(-1.0,i) * h[15-i];
	}

	c1 = 16; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet16_inv(double *valeur, int m) /* transforme inverse de Symmlet 16 d'un tableau de 2^m valeurs */
{
	double h[16] = {0.00188995033276, -0.00030292051472, -0.01495225833705, 0.00380875201389, 0.04913717967361,
					-0.02721902991706, -0.05194583810771, 0.36444189483534, 0.77718575170053, 0.48135965125838,
					-0.06127335906766, -0.14329423835081, 0.00760748732492, 0.03169508781149, -0.00054213233179,
					-0.00338241595101};
	double g[16] ;
	int i, c1, c2, resultat;

	for(i=0;i<16;i++)
	{
		g[i] = pow(-1.0,i) * h[15-i];
	}

	c1 = 16; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet18(double *valeur, int m) /* transforme de Symmlet 18 d'un tableau de 2^m valeurs */
{
	double h[18] = {0.0010694900329086053, -0.00047315449868008311, -0.010264064027633142, 0.0088592674934004842,
					0.06207778930288603, -0.018233770779395985, -0.19155083129728512, 0.035272488035271894,
					0.61733844914093583, 0.717897082764412, 0.238760914607303, -0.054568958430834071,
					0.00058346274612580684, 0.03022487885827568, -0.01152821020767923, -0.013271967781817119,
					0.00061978088898558676, 0.0014009155259146807};
	double g[18] ;
	int i, c1, c2, resultat;

	for(i=0;i<18;i++)
	{
		g[i] = pow(-1.0,i) * h[17-i];
	}

	c1 = 18; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet18_inv(double *valeur, int m) /* transforme inverse de Symmlet 18 d'un tableau de 2^m valeurs */
{
	double h[18] = {0.0010694900329086053, -0.00047315449868008311, -0.010264064027633142, 0.0088592674934004842,
					0.06207778930288603, -0.018233770779395985, -0.19155083129728512, 0.035272488035271894,
					0.61733844914093583, 0.717897082764412, 0.238760914607303, -0.054568958430834071,
					0.00058346274612580684, 0.03022487885827568, -0.01152821020767923, -0.013271967781817119,
					0.00061978088898558676, 0.0014009155259146807};
	double g[18] ;
	int i, c1, c2, resultat;

	for(i=0;i<18;i++)
	{
		g[i] = pow(-1.0,i) * h[17-i];
	}

	c1 = 18; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet20(double *valeur, int m) /* transforme de Symmlet 20 d'un tableau de 2^m valeurs */
{
	double h[20] = {-0.00045932942100465878, 5.7036083618494284e-005, 0.0045931735853118284, -0.00080435893201654491,
					-0.02035493981231129, 0.0057649120335819086, 0.049994972077376687, -0.0319900568824278,
					-0.035536740473817552, 0.38382676106708546, 0.7695100370211071, 0.47169066693843925,
					-0.070880535783243853, -0.15949427888491757, 0.011609893903711381, 0.045927239231092203,
					-0.0014653825813050513, -0.0086412992770224222, 9.5632670722894754e-005, 0.00077015980911449011};
	double g[20] ;
	int i, c1, c2, resultat;

	for(i=0;i<20;i++)
	{
		g[i] = pow(-1.0,i) * h[19-i];
	}

	c1 = 20; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet20_inv(double *valeur, int m) /* transforme inverse de Symmlet 20 d'un tableau de 2^m valeurs */
{
	double h[20] = {-0.00045932942100465878, 5.7036083618494284e-005, 0.0045931735853118284, -0.00080435893201654491,
					-0.02035493981231129, 0.0057649120335819086, 0.049994972077376687, -0.0319900568824278,
					-0.035536740473817552, 0.38382676106708546, 0.7695100370211071, 0.47169066693843925,
					-0.070880535783243853, -0.15949427888491757, 0.011609893903711381, 0.045927239231092203,
					-0.0014653825813050513, -0.0086412992770224222, 9.5632670722894754e-005, 0.00077015980911449011};
	double g[20] ;
	int i, c1, c2, resultat;

	for(i=0;i<20;i++)
	{
		g[i] = pow(-1.0,i) * h[19-i];
	}

	c1 = 20; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet22(double *valeur, int m) /* transforme de Symmlet 22 d'un tableau de 2^m valeurs */
{
	double h[22] = {0.00048926361026192387, 0.00011053509764272153, -0.0063896036664548919, -0.0020034719001093887,
					0.043000190681552281, 0.035266759564466552, -0.14460234370531561, -0.2046547944958006,
					0.23768990904924897, 0.73034354908839572, 0.57202297801008706, 0.097198394458909473,
					-0.022832651022562687, 0.069976799610734136, 0.0370374159788594, -0.024080841595864003,
					-0.0098579348287897942, 0.0065124956747714497, 0.00058835273539699145, -0.0017343662672978692,
					-3.8795655736158566e-005, 0.00017172195069934854};
	double g[22] ;
	int i, c1, c2, resultat;

	for(i=0;i<22;i++)
	{
		g[i] = pow(-1.0,i) * h[21-i];
	}

	c1 = 22; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet22_inv(double *valeur, int m) /* transforme inverse de Symmlet 22 d'un tableau de 2^m valeurs */
{
	double h[22] = {0.00048926361026192387, 0.00011053509764272153, -0.0063896036664548919, -0.0020034719001093887,
					0.043000190681552281, 0.035266759564466552, -0.14460234370531561, -0.2046547944958006,
					0.23768990904924897, 0.73034354908839572, 0.57202297801008706, 0.097198394458909473,
					-0.022832651022562687, 0.069976799610734136, 0.0370374159788594, -0.024080841595864003,
					-0.0098579348287897942, 0.0065124956747714497, 0.00058835273539699145, -0.0017343662672978692,
					-3.8795655736158566e-005, 0.00017172195069934854};
	double g[22] ;
	int i, c1, c2, resultat;

	for(i=0;i<22;i++)
	{
		g[i] = pow(-1.0,i) * h[21-i];
	}

	c1 = 22; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}


int symmlet24(double *valeur, int m) /* transforme de Symmlet 24 d'un tableau de 2^m valeurs */
{
	double h[24] = {-0.00017906658697508691, -1.8158078862617515e-005, 0.0023502976141834648, 0.00030764779631059454,
					-0.014589836449234145, -0.0026043910313322326, 0.057804179445505657, 0.01530174062247884,
					-0.17037069723886492, -0.07833262231634322, 0.46274103121927235, 0.76347909778365719,
					0.39888597239022, -0.022162306170337816, -0.035848830736954392, 0.049179318299660837,
					0.0075537806116804775, -0.024220722675013445, -0.0014089092443297553, 0.007414965517654251,
					0.00018021409008538188, -0.0013497557555715387, -1.1353928041541452e-005, 0.00011196719424656033};
	double g[24] ;
	int i, c1, c2, resultat;

	for(i=0;i<24;i++)
	{
		g[i] = pow(-1.0,i) * h[23-i];
	}

	c1 = 24; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int symmlet24_inv(double *valeur, int m) /* transforme inverse de Symmlet 24 d'un tableau de 2^m valeurs */
{
	double h[24] = {-0.00017906658697508691, -1.8158078862617515e-005, 0.0023502976141834648, 0.00030764779631059454,
					-0.014589836449234145, -0.0026043910313322326, 0.057804179445505657, 0.01530174062247884,
					-0.17037069723886492, -0.07833262231634322, 0.46274103121927235, 0.76347909778365719,
					0.39888597239022, -0.022162306170337816, -0.035848830736954392, 0.049179318299660837,
					0.0075537806116804775, -0.024220722675013445, -0.0014089092443297553, 0.007414965517654251,
					0.00018021409008538188, -0.0013497557555715387, -1.1353928041541452e-005, 0.00011196719424656033};
	double g[24] ;
	int i, c1, c2, resultat;

	for(i=0;i<24;i++)
	{
		g[i] = pow(-1.0,i) * h[23-i];
	}

	c1 = 24; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}


/* Transforme en ondelettes Coiflet
   =================================

  Le principe de l'algorithme de transforme en ondelette Coiflet est identique
   celui implment pour les ondelettes de Daubechies. Seul les coefficients des
  fonction d'chelle et d'ondelette sont modifis.

*/


/* test avec d'autres coefficients */






/* fin du test */






int coiflet6(double *valeur, int m) /* transforme de Coiflet 6 d'un tableau de 2^m valeurs */
{
	double h[6] = {-0.07273261951285, 0.33789766245781, 0.85257202021226, 0.38486484686420, -0.07273261951285, -0.01565572813546};
	double g[6] ;
	int i, c1, c2, resultat;

	for(i=0;i<6;i++)
	{
		g[i] = pow(-1.0,i) * h[5-i];
	}

	c1 = 6; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet6_inv(double *valeur, int m) /* transforme inverse de Coiflet 6 d'un tableau de 2^m valeurs */
{
	double h[6] = {-0.07273261951285, 0.33789766245781, 0.85257202021226, 0.38486484686420, -0.07273261951285, -0.01565572813546};
	double g[6] ;
	int i, c1, c2, resultat;

	for(i=0;i<6;i++)
	{
		g[i] = pow(-1.0,i) * h[5-i];
	}

	c1 = 6; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}


int coiflet12(double *valeur, int m) /* transforme de Coiflet 12 d'un tableau de 2^m valeurs */
{
	double h[12] = {0.01638733646360, -0.04146493678197, -0.06737255472230, 0.38611006682309, 0.81272363544961,
					0.41700518442378, -0.07648859907869, -0.05943441864675, 0.02368017194645, 0.00561143481942,
					-0.00182320887071, -0.00072054944537};
	double g[12] ;
	int i, c1, c2, resultat;

	for(i=0;i<12;i++)
	{
		g[i] = pow(-1.0,i) * h[11-i];
	}

	c1 = 12; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet12_inv(double *valeur, int m) /* transforme inverse de Coiflet 12 d'un tableau de 2^m valeurs */
{
	double h[12] = {0.01638733646360, -0.04146493678197, -0.06737255472230, 0.38611006682309, 0.81272363544961,
					0.41700518442378, -0.07648859907869, -0.05943441864675, 0.02368017194645, 0.00561143481942,
					-0.00182320887071, -0.00072054944537};
	double g[12] ;
	int i, c1, c2, resultat;

	for(i=0;i<12;i++)
	{
		g[i] = pow(-1.0,i) * h[11-i];
	}

	c1 = 12; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet18(double *valeur, int m) /* transforme de Coiflet 18 d'un tableau de 2^m valeurs */
{
	double h[18] = {-0.00379351286449,0.00778259642733,0.02345269614184, -0.06577191128186, -0.06112339000267,
					0.40517690240962, 0.79377722262562, 0.42848347637762, -0.07179982161931, -0.08230192710689,
					0.03455502757306, 0.01588054486362, -0.00900797613666, -0.00257451768875, 0.00111751877089,
					0.00046621696011, -0.00007098330314, -0.00003459977284};
	double g[18] ;
	int i, c1, c2, resultat;

	for(i=0;i<18;i++)
	{
		g[i] = pow(-1.0,i) * h[17-i];
	}

	c1 = 18; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet18_inv(double *valeur, int m) /* transforme inverse de Coiflet 18 d'un tableau de 2^m valeurs */
{
	double h[18] = {-0.00379351286449,0.00778259642733,0.02345269614184, -0.06577191128186, -0.06112339000267,
					0.40517690240962, 0.79377722262562, 0.42848347637762, -0.07179982161931, -0.08230192710689,
					0.03455502757306, 0.01588054486362, -0.00900797613666, -0.00257451768875, 0.00111751877089,
					0.00046621696011, -0.00007098330314, -0.00003459977284};
	double g[18] ;
	int i, c1, c2, resultat;

	for(i=0;i<18;i++)
	{
		g[i] = pow(-1.0,i) * h[17-i];
	}

	c1 = 18; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet24(double *valeur, int m) /* transforme de Coiflet 24 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[24];
	double h[24] = { 0.00089231366858, -0.00162949201260, -0.00734616632764, 0.01606894396478, 0.02668230015605, -0.08126669968088,
					-0.05607731331675, 0.41530840703043, 0.78223893092050, 0.43438605649147, -0.06662747426343, -0.09622044203399,
					0.03933442712334, 0.02508226184486, -0.01521173152795, -0.00565828668661, 0.00375143615728, 0.00126656192930,
					-0.00058902075624, -0.00025997455249, 0.00006233903446, 0.00003122987587, -0.00000325968024, -0.00000178498500};
    
	for(i=0;i<24;i++)
	{
		g[i] = pow(-1.0,i) * h[23-i]; /* 23 = 24 - 1 */
	}

	c1 = 24; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);

	
}


int coiflet24_inv(double *valeur, int m) /* transforme inverse de Coiflet 24 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[24];
	double h[24] = { 0.00089231366858, -0.00162949201260, -0.00734616632764, 0.01606894396478, 0.02668230015605, -0.08126669968088,
					-0.05607731331675, 0.41530840703043, 0.78223893092050, 0.43438605649147, -0.06662747426343, -0.09622044203399,
					0.03933442712334, 0.02508226184486, -0.01521173152795, -0.00565828668661, 0.00375143615728, 0.00126656192930,
					-0.00058902075624, -0.00025997455249, 0.00006233903446, 0.00003122987587, -0.00000325968024, -0.00000178498500};
    
	for(i=0;i<24;i++)
	{
		g[i] = pow(-1.0,i) * h[23-i]; /* 23 = 24 - 1 */
	}

	c1 = 24; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);		
}

int coiflet30(double *valeur, int m) /* transforme de Coiflet 30 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[30];
	double h[30] = {-0.00021208083980379827, 0.00035858968789573785, 0.0021782363581090178, -0.004159358781386048, 
					-0.010131117519849788, 0.023408156785839195, 0.02816802897093635, -0.091920010559696244,
					-0.052043163176243773, 0.42156620669085149, 0.77428960365295618, 0.43799162617183712,
					-0.062035963962903569, -0.10557420870333893, 0.041289208750181702, 0.032683574267111833,
					-0.019761778942572639, -0.0091642311624818458, 0.0067641854480530832, 0.0024333732126576722,
					-0.0016628637020130838, -0.00063813134304511142, 0.00030225958181306315, 0.00014054114970203437,
					-4.1340432272512511e-005, -2.1315026809955787e-005, 3.7346551751414047e-006, 2.0637618513646814e-006,
					-1.6744288576823017e-007, -9.517657273819165e-008};
    
	for(i=0;i<30;i++)
	{
		g[i] = pow(-1.0,i) * h[29-i]; /* 29 = 30 - 1 */
	}

	c1 = 30; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);

	
}


int coiflet30_inv(double *valeur, int m) /* transforme inverse de Coiflet 30 d'un tableau de 2^m valeurs */
{
	int i, c1, c2, resultat;
	double g[30];
	double h[30] = {-0.00021208083980379827, 0.00035858968789573785, 0.0021782363581090178, -0.004159358781386048, 
					-0.010131117519849788, 0.023408156785839195, 0.02816802897093635, -0.091920010559696244,
					-0.052043163176243773, 0.42156620669085149, 0.77428960365295618, 0.43799162617183712,
					-0.062035963962903569, -0.10557420870333893, 0.041289208750181702, 0.032683574267111833,
					-0.019761778942572639, -0.0091642311624818458, 0.0067641854480530832, 0.0024333732126576722,
					-0.0016628637020130838, -0.00063813134304511142, 0.00030225958181306315, 0.00014054114970203437,
					-4.1340432272512511e-005, -2.1315026809955787e-005, 3.7346551751414047e-006, 2.0637618513646814e-006,
					-1.6744288576823017e-007, -9.517657273819165e-008};
    
	for(i=0;i<30;i++)
	{
		g[i] = pow(-1.0,i) * h[29-i]; /* 29 = 30 - 1 */
	}

	c1 = 30; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);		
}


int coiflet36(double *valeur, int m) /* transforme de Coiflet 36 d'un tableau de 2^m valeurs */
{
	double h[36] = {5.077548784E-05, -8.117002627E-05, -6.246130439E-04, 1.091624712E-03, 3.539019872E-03, -7.029406391E-03,
					-1.223157779E-02, 2.964577289E-02, 2.878611435E-02, -9.967300205E-02, -4.876407218E-02, 4.258195450E-01,
					7.684032576E-01, 4.404011911E-01, -5.810891797E-02, -1.122608080E-01, 4.185249068E-02, 3.888132625E-02,
					-2.295015328E-02, -1.265006791E-02, 9.591090176E-03, 3.857658271E-03, -3.073939507E-03, -1.157435013E-03,
					7.698547308E-04, 3.252223590E-04, -1.545771993E-04, -7.528004307E-05, 2.473655933E-05, 1.313985135E-05,
					-2.924385560E-06, -1.659619295E-06, 2.255997853E-07, 1.350324499E-07, -8.487143396E-09, -5.309088417E-09};
	double g[36] ;
	int i, c1, c2, resultat;

	for(i=0;i<36;i++)
	{
		g[i] = pow(-1.0,i) * h[35-i];
	}

	c1 = 36; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 64; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet36_inv(double *valeur, int m) /* transforme inverse de Coiflet 36 d'un tableau de 2^m valeurs */
{
	double h[36] = {5.077548784E-05, -8.117002627E-05, -6.246130439E-04, 1.091624712E-03, 3.539019872E-03, -7.029406391E-03,
					-1.223157779E-02, 2.964577289E-02, 2.878611435E-02, -9.967300205E-02, -4.876407218E-02, 4.258195450E-01,
					7.684032576E-01, 4.404011911E-01, -5.810891797E-02, -1.122608080E-01, 4.185249068E-02, 3.888132625E-02,
					-2.295015328E-02, -1.265006791E-02, 9.591090176E-03, 3.857658271E-03, -3.073939507E-03, -1.157435013E-03,
					7.698547308E-04, 3.252223590E-04, -1.545771993E-04, -7.528004307E-05, 2.473655933E-05, 1.313985135E-05,
					-2.924385560E-06, -1.659619295E-06, 2.255997853E-07, 1.350324499E-07, -8.487143396E-09, -5.309088417E-09};
	double g[36] ;
	int i, c1, c2, resultat;

	for(i=0;i<36;i++)
	{
		g[i] = pow(-1.0,i) * h[35-i];
	}

	c1 = 36; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 64; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}


int coiflet42(double *valeur, int m) /* transforme de Coiflet 42 d'un tableau de 2^m valeurs */
{
	double h[42] = {-1.222225062E-05, 1.871135500E-05, 1.751021678E-04, -2.872023754E-04, -1.169314429E-03, 2.105772041E-03,
					4.829446561E-03, -9.938895269E-03, -1.380255424E-02, 3.491050510E-02, 2.893704198E-02, -1.055561682E-01,
					-4.603339704E-02, 4.288888072E-01, 7.638153654E-01, 4.421374614E-01, -5.475124165E-02, -1.172935710E-01,
					4.170535760E-02, 4.399304616E-02, -2.515425757E-02, -1.594684682E-02, 1.205233824E-02, 5.431316443E-03,
					-4.617842130E-03, -1.801537283E-03, 1.434741857E-03, 5.794994482E-04, -3.690668287E-04, -1.678172122E-04,
					7.971050026E-05, 4.043048242E-05, -1.423563698E-05, -7.771243547E-06, 2.002078050E-06, 1.157976907E-06,
					-2.069320524E-07, -1.255091319E-07, 1.393510389E-08, 8.796593385E-09, -4.578334068E-10, -2.990566232E-10};
	double g[42] ;
	int i, c1, c2, resultat;

	for(i=0;i<42;i++)
	{
		g[i] = pow(-1.0,i) * h[41-i];
	}

	c1 = 42; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 64; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet42_inv(double *valeur, int m) /* transforme inverse de Coiflet 42 d'un tableau de 2^m valeurs */
{
	double h[42] = {-1.222225062E-05, 1.871135500E-05, 1.751021678E-04, -2.872023754E-04, -1.169314429E-03, 2.105772041E-03,
					4.829446561E-03, -9.938895269E-03, -1.380255424E-02, 3.491050510E-02, 2.893704198E-02, -1.055561682E-01,
					-4.603339704E-02, 4.288888072E-01, 7.638153654E-01, 4.421374614E-01, -5.475124165E-02, -1.172935710E-01,
					4.170535760E-02, 4.399304616E-02, -2.515425757E-02, -1.594684682E-02, 1.205233824E-02, 5.431316443E-03,
					-4.617842130E-03, -1.801537283E-03, 1.434741857E-03, 5.794994482E-04, -3.690668287E-04, -1.678172122E-04,
					7.971050026E-05, 4.043048242E-05, -1.423563698E-05, -7.771243547E-06, 2.002078050E-06, 1.157976907E-06,
					-2.069320524E-07, -1.255091319E-07, 1.393510389E-08, 8.796593385E-09, -4.578334068E-10, -2.990566232E-10};
	double g[42] ;
	int i, c1, c2, resultat;

	for(i=0;i<42;i++)
	{
		g[i] = pow(-1.0,i) * h[41-i];
	}

	c1 = 42; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 64; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}


int coiflet48(double *valeur, int m) /* transforme de Coiflet 48 d'un tableau de 2^m valeurs */
{
	double h[48] = {2.954336521E-06, -4.368264820E-06, -4.829631521E-05, 7.547367838E-05, 3.712949956E-04, -6.235604475E-04,
					-1.783260009E-03, 3.300825011E-03, 5.994849192E-03, -1.274237063E-02, -1.497846208E-02, 3.937203788E-02,
					2.882862176E-02, -1.101699770E-01, -4.371898337E-02, 4.312098156E-01, 7.601133020E-01, 4.434425498E-01,
					-5.186074316E-02, -1.212111682E-01, 4.118580668E-02, 4.825237109E-02, -2.665671054E-02, -1.898524470E-02,
					1.411747008E-02, 7.065827011E-03, -6.156659548E-03, -2.544003710E-03, 2.235649422E-03, 8.967760631E-04,
					-6.871716433E-04, -2.977789322E-04, 1.816928765E-04, 8.754452092E-05, -4.147478607E-05, -2.180200077E-05,
					8.031502995E-06, 4.496936444E-06, -1.275454300E-06, -7.515021559E-07, 1.589351722E-07, 9.772418508E-08,
					-1.454000853E-08, -9.271205592E-09, 8.669995082E-10, 5.704810334E-10, -2.525423494E-11, -1.707989595E-11};
	double g[48] ;
	int i, c1, c2, resultat;

	for(i=0;i<48;i++)
	{
		g[i] = pow(-1.0,i) * h[47-i];
	}

	c1 = 48; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 64; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet48_inv(double *valeur, int m) /* transforme inverse de Coiflet 48 d'un tableau de 2^m valeurs */
{
	double h[48] = {2.954336521E-06, -4.368264820E-06, -4.829631521E-05, 7.547367838E-05, 3.712949956E-04, -6.235604475E-04,
					-1.783260009E-03, 3.300825011E-03, 5.994849192E-03, -1.274237063E-02, -1.497846208E-02, 3.937203788E-02,
					2.882862176E-02, -1.101699770E-01, -4.371898337E-02, 4.312098156E-01, 7.601133020E-01, 4.434425498E-01,
					-5.186074316E-02, -1.212111682E-01, 4.118580668E-02, 4.825237109E-02, -2.665671054E-02, -1.898524470E-02,
					1.411747008E-02, 7.065827011E-03, -6.156659548E-03, -2.544003710E-03, 2.235649422E-03, 8.967760631E-04,
					-6.871716433E-04, -2.977789322E-04, 1.816928765E-04, 8.754452092E-05, -4.147478607E-05, -2.180200077E-05,
					8.031502995E-06, 4.496936444E-06, -1.275454300E-06, -7.515021559E-07, 1.589351722E-07, 9.772418508E-08,
					-1.454000853E-08, -9.271205592E-09, 8.669995082E-10, 5.704810334E-10, -2.525423494E-11, -1.707989595E-11};
	double g[48] ;
	int i, c1, c2, resultat;

	for(i=0;i<48;i++)
	{
		g[i] = pow(-1.0,i) * h[47-i];
	}

	c1 = 48; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 64; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet54(double *valeur, int m) /* transforme de Coiflet 54 d'un tableau de 2^m valeurs */
{
	double h[54] = {-7.164920431E-07, 1.029320067E-06, 1.315888565E-05, -1.978720443E-05, -1.144339528E-04, 1.822848597E-04,
					6.264730321E-04, -1.075458273E-03, -2.421241674E-03, 4.597056425E-03, 7.022340460E-03, -1.537664963E-02,
					-1.586022389E-02, 4.318172761E-02, 2.857266756E-02, -1.138835082E-01, -4.172611021E-02, 4.330267511E-01,
					7.570455234E-01, 4.444578932E-01, -4.934886629E-02, -1.243455895E-01, 4.047376746E-02, 5.184461569E-02,
					-2.766123950E-02, -2.175455351E-02, 1.581871582E-02, 8.702757446E-03, -7.614042448E-03, -3.357674527E-03,
					3.113227788E-03, 1.269690925E-03, -1.095745628E-03, -4.627290855E-04, 3.369138693E-04, 1.554135213E-04,
					-9.135595509E-05, -4.613708198E-05, 2.177639641E-05, 1.181440945E-05, -4.488111475E-06, -2.572383574E-06,
					7.802480329E-07, 4.679584769E-07, -1.109667018E-07, -6.916547041E-08, 1.237525662E-08, 7.974005887E-09,
					-1.013627569E-09, -6.723464415E-10, 5.417100964E-11, 3.686179736E-11, -1.416273551E-12, -9.858437261E-13};
	double g[54] ;
	int i, c1, c2, resultat;

	for(i=0;i<54;i++)
	{
		g[i] = pow(-1.0,i) * h[53-i];
	}

	c1 = 54; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 64; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet54_inv(double *valeur, int m) /* transforme inverse de Coiflet 54 d'un tableau de 2^m valeurs */
{
	double h[54] = {-7.164920431E-07, 1.029320067E-06, 1.315888565E-05, -1.978720443E-05, -1.144339528E-04, 1.822848597E-04,
					6.264730321E-04, -1.075458273E-03, -2.421241674E-03, 4.597056425E-03, 7.022340460E-03, -1.537664963E-02,
					-1.586022389E-02, 4.318172761E-02, 2.857266756E-02, -1.138835082E-01, -4.172611021E-02, 4.330267511E-01,
					7.570455234E-01, 4.444578932E-01, -4.934886629E-02, -1.243455895E-01, 4.047376746E-02, 5.184461569E-02,
					-2.766123950E-02, -2.175455351E-02, 1.581871582E-02, 8.702757446E-03, -7.614042448E-03, -3.357674527E-03,
					3.113227788E-03, 1.269690925E-03, -1.095745628E-03, -4.627290855E-04, 3.369138693E-04, 1.554135213E-04,
					-9.135595509E-05, -4.613708198E-05, 2.177639641E-05, 1.181440945E-05, -4.488111475E-06, -2.572383574E-06,
					7.802480329E-07, 4.679584769E-07, -1.109667018E-07, -6.916547041E-08, 1.237525662E-08, 7.974005887E-09,
					-1.013627569E-09, -6.723464415E-10, 5.417100964E-11, 3.686179736E-11, -1.416273551E-12, -9.858437261E-13};
	double g[54] ;
	int i, c1, c2, resultat;

	for(i=0;i<54;i++)
	{
		g[i] = pow(-1.0,i) * h[53-i];
	}

	c1 = 54; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 64; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}


int coiflet60(double *valeur, int m) /* transforme de Coiflet 60 d'un tableau de 2^m valeurs */
{
	double h[60] = {1.742367480E-07, -2.442764865E-07, -3.551205539E-06, 5.173962608E-06, 3.445969323E-05, -5.264472186E-05,
					-2.117741365E-04, 3.434550262E-04, 9.249399604E-04, -1.620778109E-03, -3.053992494E-03, 5.937373266E-03,
					7.917157068E-03, -1.782044578E-02, -1.652151127E-02, 4.646274705E-02, 2.823291274E-02, -1.169360705E-01,
					-3.998711302E-02, 4.344881822E-01, 7.544501095E-01, 4.452691977E-01, -4.714526254E-02, -1.269091043E-01,
					3.966834928E-02, 5.490896400E-02, -2.831006394E-02, -2.426732868E-02, 1.720591250E-02, 1.030537800E-02,
					-8.953207287E-03, -4.218113885E-03, 4.020222791E-03, 1.689969793E-03, -1.574858223E-03, -6.598662532E-04,
					5.451673709E-04, 2.436317307E-04, -1.685798343E-04, -8.202162256E-05, 4.672498135E-05, 2.454191012E-05,
					-1.153105839E-05, -6.434329489E-06, 2.497202791E-06, 1.462445032E-06, -4.657624401E-07, -2.840907584E-07,
					7.315542759E-08, 4.620903057E-08, -9.396332747E-09, -6.118910133E-09, 9.467830637E-10, 6.333121950E-10,
					-7.012920333E-11, -4.804052212E-11, 3.393464738E-12, 2.374617931E-12, -8.044508599E-14,-5.737961267E-14};
	double g[60] ;
	int i, c1, c2, resultat;

	for(i=0;i<60;i++)
	{
		g[i] = pow(-1.0,i) * h[59-i];
	}

	c1 = 60; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 64; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int coiflet60_inv(double *valeur, int m) /* transforme inverse de Coiflet 60 d'un tableau de 2^m valeurs */
{
	double h[60] = {1.742367480E-07, -2.442764865E-07, -3.551205539E-06, 5.173962608E-06, 3.445969323E-05, -5.264472186E-05,
					-2.117741365E-04, 3.434550262E-04, 9.249399604E-04, -1.620778109E-03, -3.053992494E-03, 5.937373266E-03,
					7.917157068E-03, -1.782044578E-02, -1.652151127E-02, 4.646274705E-02, 2.823291274E-02, -1.169360705E-01,
					-3.998711302E-02, 4.344881822E-01, 7.544501095E-01, 4.452691977E-01, -4.714526254E-02, -1.269091043E-01,
					3.966834928E-02, 5.490896400E-02, -2.831006394E-02, -2.426732868E-02, 1.720591250E-02, 1.030537800E-02,
					-8.953207287E-03, -4.218113885E-03, 4.020222791E-03, 1.689969793E-03, -1.574858223E-03, -6.598662532E-04,
					5.451673709E-04, 2.436317307E-04, -1.685798343E-04, -8.202162256E-05, 4.672498135E-05, 2.454191012E-05,
					-1.153105839E-05, -6.434329489E-06, 2.497202791E-06, 1.462445032E-06, -4.657624401E-07, -2.840907584E-07,
					7.315542759E-08, 4.620903057E-08, -9.396332747E-09, -6.118910133E-09, 9.467830637E-10, 6.333121950E-10,
					-7.012920333E-11, -4.804052212E-11, 3.393464738E-12, 2.374617931E-12, -8.044508599E-14,-5.737961267E-14};
	double g[60] ;
	int i, c1, c2, resultat;

	for(i=0;i<60;i++)
	{
		g[i] = pow(-1.0,i) * h[59-i];
	}

	c1 = 60; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 64; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);
}



/* Transforme en ondelettes biorthogonale
   =======================================

  Le principe de l'algorithme de transforme en ondelette biorthogonale est identique
   celui implment pour les ondelettes de Daubechies. Seul les coefficients des
  fonction d'chelle et d'ondelette sont modifis.

*/

int bio13(double *valeur, int m) /* transforme biorthogonale 1.3 d'un tableau de 2^m valeurs */
{
	double h[6] = {0.0, 0.0, 0.7071067812, 0.7071067812, 0.0, 0.0};
	double g[6] = {-0.0883883476, -0.0883883476, 0.7071067812,
					-0.7071067812, 0.0883883476, 0.0883883476};
	int c1, c2, resultat;

	c1 = 6; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */
	
	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
	
}


int bio13_inv(double *valeur, int m) /* transforme biorthogonale inverse 1.3 d'un tableau de 2^m valeurs */
{
	double h[6] = {-0.0883883476, 0.0883883476, 0.7071067812, 0.7071067812,
					0.0883883476, -0.0883883476};
	double g[6] = {0.0, 0.0, 0.7071067812, -0.7071067812, 0.0, 0.0};
	int c1, c2, resultat;

	c1 = 6; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}


int bio15(double *valeur, int m) /* transforme biorthogonale 1.5 d'un tableau de 2^m valeurs */
{
	double h[10] = {0.0, 0.0, 0.0, 0.0, 0.7071067812, 0.7071067812, 0.0, 0.0, 0.0, 0.0};
	double g[10] = {0.0165728152, 0.0165728152, -0.1215339780, -0.1215339780, 0.7071067812,
					-0.7071067812, 0.1215339780, 0.1215339780, -0.0165728152, -0.0165728152};
	int c1, c2, resultat;

	c1 = 10; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio15_inv(double *valeur, int m) /* transforme biorthogonale inverse 1.5 d'un tableau de 2^m valeurs */
{
	double h[10] = {0.0165728152, -0.0165728152, -0.1215339780, 0.1215339780, 0.7071067812,
					0.7071067812, 0.1215339780, -0.1215339780, -0.0165728152, 0.0165728152};
	double g[10] = {0.0, 0.0, 0.0, 0.0, 0.7071067812, -0.7071067812, 0.0, 0.0, 0.0, 0.0};

	int c1, c2, resultat;

	c1 = 10; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}


int bio22(double *valeur, int m) /* transforme biorthogonale 2.2 d'un tableau de 2^m valeurs */
{
	double h[6] = {0.0, 0.35355339059327379, 0.70710678118654757, 0.35355339059327379, 0.0, 0.0};
	double g[6] = {0.0, 0.17677669529663689, 0.35355339059327379, -1.0606601717798214, 0.35355339059327379, 0.17677669529663689};
	int c1, c2, resultat;

	c1 = 6; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio22_inv(double *valeur, int m) /* transforme biorthogonale inverse 2.2 d'un tableau de 2^m valeurs */
{
	double h[6] = {-0.17677669529663689, 0.35355339059327379, 1.0606601717798214, 0.35355339059327379, -0.17677669529663689, 0.0};
	double g[6] = { 0.0, 0.0, 0.35355339059327379, -0.70710678118654757, 0.35355339059327379, 0.0};

	int c1, c2, resultat;

	c1 = 6; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}


int bio24(double *valeur, int m) /* transforme biorthogonale 2.4 d'un tableau de 2^m valeurs */
{
	double h[10] = {0.0, 0.0, 0.0, 0.35355339059327379, 0.70710678118654757, 0.35355339059327379, 0.0, 0.0, 0.0, 0.0};
	double g[10] = {0.0, -0.033145630368119419, -0.066291260736238838, 0.17677669529663689, 0.4198446513295126,
					-0.99436891104358249, 0.4198446513295126, 0.17677669529663689, -0.066291260736238838, -0.033145630368119419};
	int c1, c2, resultat;

	c1 = 10; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio24_inv(double *valeur, int m) /* transforme biorthogonale inverse 2.4 d'un tableau de 2^m valeurs */
{
	double h[10] = {0.033145630368119419, -0.066291260736238838, -0.17677669529663689, 0.4198446513295126, 
					0.99436891104358249, 0.4198446513295126, -0.17677669529663689, -0.066291260736238838, 0.033145630368119419, 0.0};
	double g[10] = {0.0, 0.0, 0.0, 0.0, 0.35355339059327379, -0.70710678118654757, 0.35355339059327379, 0.0, 0.0, 0.0};

	int c1, c2, resultat;

	c1 = 10; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}


int bio26(double *valeur, int m) /* transforme biorthogonale 2.6 d'un tableau de 2^m valeurs */
{
	double h[14] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.3535533906, 0.7071067812, 0.3535533906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
	double g[14] = {0.0, 0.0069053397, 0.0138106793, -0.0469563097, -0.1077232987, 0.1698713556, 0.4474660100,
					-0.9667475524, 0.4474660100, 0.1698713556, -0.1077232987, -0.0469563097, 0.0138106793, 0.0069053397};
	int c1, c2, resultat;

	c1 = 14; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio26_inv(double *valeur, int m) /* transforme biorthogonale inverse 2.6 d'un tableau de 2^m valeurs */
{
	double h[14] = {-0.0069053397, 0.0138106793, 0.0469563097, -0.1077232987, -0.1698713556,
					0.4474660100, 0.9667475524, 0.4474660100, -0.1698713556, -0.1077232987,
					0.0469563097, 0.0138106793, -0.0069053397, 0.0};
	double g[14] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3535533906, -0.7071067812, 0.3535533906, 0.0, 0.0, 0.0, 0.0, 0.0};

	int c1, c2, resultat;

	c1 = 14; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int bio28(double *valeur, int m) /* transforme biorthogonale 2.8 d'un tableau de 2^m valeurs */
{
	double h[18] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3535533906, 0.7071067812,
					0.3535533906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
	double g[18] = {0.0, -0.0015105431, -0.0030210861, 0.0129475119, 0.0289161098, -0.0529984819, -0.1349130736,
					0.1638291834, 0.4625714405, -0.9516421219, 0.4625714405, 0.1638291834, -0.1349130736,
					-0.0529984819, 0.0289161098, 0.0129475119, -0.0030210861, -0.0015105431};
	int c1, c2, resultat;

	c1 = 18; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio28_inv(double *valeur, int m) /* transforme biorthogonale inverse 2.8 d'un tableau de 2^m valeurs */
{
	double h[18] = {0.0015105431, -0.0030210861, -0.0129475119, 0.0289161098, 0.0529984819, -0.1349130736,
					-0.1638291834, 0.4625714405, 0.9516421219, 0.4625714405, -0.1638291834, -0.1349130736,
					0.0529984819, 0.0289161098, -0.0129475119, -0.0030210861, 0.0015105431, 0.0};
	double g[18] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3535533906, -0.7071067812,
					0.3535533906, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};

	int c1, c2, resultat;

	c1 = 18; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int bio31(double *valeur, int m) /* transforme biorthogonale 3.1 d'un tableau de 2^m valeurs */
{
	double h[4] = {0.1767766953, 0.5303300859, 0.5303300859, 0.1767766953};
	double g[4] = {-0.3535533906, -1.0606601718, 1.0606601718, 0.3535533906};
	int c1, c2, resultat;

	c1 = 4; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 4; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio31_inv(double *valeur, int m) /* transforme biorthogonale inverse 3.1 d'un tableau de 2^m valeurs */
{
	double h[4] = {-0.3535533906, 1.0606601718, 1.0606601718, -0.3535533906};
	double g[4] = {0.1767766953, -0.5303300859, 0.5303300859, -0.1767766953};

	int c1, c2, resultat;

	c1 = 4; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 4; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int bio33(double *valeur, int m) /* transforme biorthogonale 3.3 d'un tableau de 2^m valeurs */
{
	double h[8] = {0.0, 0.0, 0.1767766953, 0.5303300859, 0.5303300859, 0.1767766953, 0.0, 0.0};
	double g[8] = {0.0662912607, 0.1988737822, -0.1546796084, -0.9943689110, 0.9943689110,
					0.1546796084, -0.1988737822, -0.0662912607};
	int c1, c2, resultat;

	c1 = 8; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio33_inv(double *valeur, int m) /* transforme biorthogonale inverse 3.3 d'un tableau de 2^m valeurs */
{
	double h[8] = {0.0662912607, -0.1988737822, -0.1546796084, 0.9943689110, 0.9943689110,
					-0.1546796084, -0.1988737822, 0.0662912607};
	double g[8] = {0.0, 0.0, 0.1767766953, -0.5303300859, 0.5303300859,
					-0.1767766953, 0.0, 0.0};

	int c1, c2, resultat;

	c1 = 8; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 8; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int bio35(double *valeur, int m) /* transforme biorthogonale 3.5 d'un tableau de 2^m valeurs */
{
	double h[12] = {0.0, 0.0, 0.0, 0.0, 0.1767766953, 0.5303300859, 0.5303300859,
					0.1767766953, 0.0, 0.0, 0.0, 0.0};
	double g[12] = {-0.0138106793, -0.0414320380, 0.0524805814, 0.2679271788, -0.0718155325, -0.9667475524,
					0.9667475524, 0.0718155325, -0.2679271788, -0.0524805814, 0.0414320380, 0.0138106793};
	int c1, c2, resultat;

	c1 = 12; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio35_inv(double *valeur, int m) /* transforme biorthogonale inverse 3.5 d'un tableau de 2^m valeurs */
{
	double h[12] = {-0.0138106793, 0.0414320380, 0.0524805814, -0.2679271788, -0.0718155325, 0.9667475524,
					0.9667475524, -0.0718155325, -0.2679271788, 0.0524805814, 0.0414320380, -0.0138106793};
	double g[12] = {0.0, 0.0, 0.0, 0.0, 0.1767766953, -0.5303300859, 0.5303300859,
					-0.1767766953, 0.0, 0.0, 0.0, 0.0};

	int c1, c2, resultat;

	c1 = 12; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int bio37(double *valeur, int m) /* transforme biorthogonale 3.7 d'un tableau de 2^m valeurs */
{
	double h[16] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.1767766953, 0.5303300859,
					0.5303300859, 0.1767766953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
	double g[16] = {0.0030210861, 0.0090632583, -0.0168317654, -0.0746639851, 0.0313329787,
					0.3011591259, -0.0264992409, -0.9516421219, 0.9516421219, 0.0264992409,
					-0.3011591259, -0.0313329787, 0.0746639851, 0.0168317654, -0.0090632583, -0.0030210861};
	int c1, c2, resultat;

	c1 = 16; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio37_inv(double *valeur, int m) /* transforme biorthogonale inverse 3.7 d'un tableau de 2^m valeurs */
{
	double h[16] = {0.0030210861, -0.0090632583, -0.0168317654, 0.0746639851, 0.0313329787,
					-0.3011591259, -0.0264992409, 0.9516421219, 0.9516421219, -0.0264992409,
					-0.3011591259, 0.0313329787, 0.0746639851, -0.0168317654, -0.0090632583, 0.0030210861};
	double g[16] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.1767766953, -0.5303300859, 0.5303300859,
					-0.1767766953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};

	int c1, c2, resultat;

	c1 = 16; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int bio39(double *valeur, int m) /* transforme biorthogonale 3.9 d'un tableau de 2^m valeurs */
{
	double h[20] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.1767766953, 0.5303300859, 0.5303300859,
					0.1767766953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
	double g[20] = {-0.0006797444, -0.0020392331, 0.0050603192, 0.0206189126, -0.0141127879, -0.0991347825,
					0.0123001363, 0.3201919684, 0.0020500227, -0.9421257007, 0.9421257007, -0.0020500227,
					-0.3201919684, -0.0123001363, 0.0991347825, 0.0141127879, -0.0206189126, -0.0050603192,
					 0.0020392331, 0.0006797444};
	int c1, c2, resultat;

	c1 = 20; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio39_inv(double *valeur, int m) /* transforme biorthogonale inverse 3.9 d'un tableau de 2^m valeurs */
{
	double h[20] = {-0.0006797444, 0.0020392331, 0.0050603192, -0.0206189126, -0.0141127879, 0.0991347825, 0.0123001363,
					-0.3201919684, 0.0020500227, 0.9421257007, 0.9421257007, 0.0020500227, -0.3201919684, 0.0123001363,
					 0.0991347825, -0.0141127879, -0.0206189126, 0.0050603192, 0.0020392331, -0.0006797444};
	double g[20] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.1767766953, -0.5303300859, 0.5303300859,
					-0.1767766953, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};

	int c1, c2, resultat;

	c1 = 20; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int bio44(double *valeur, int m) /* transforme biorthogonale 4.4 d'un tableau de 2^m valeurs */
{
	double h[10] = {0.0, -0.0645388826, -0.0406894176, 0.4180922732, 0.7884856164, 0.4180922732,
					-0.0406894176, -0.0645388826, 0.0, 0.0};
	double g[10] = {0.0, -0.0378284555, -0.0238494650, 0.1106244044, 0.3774028556, -0.8526986790,
					0.3774028556, 0.1106244044, -0.0238494650, -0.0378284555};
	int c1, c2, resultat;

	c1 = 10; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio44_inv(double *valeur, int m) /* transforme biorthogonale inverse 4.4 d'un tableau de 2^m valeurs */
{
	double h[10] = {0.0378284555, -0.0238494650, -0.1106244044, 0.3774028556, 0.8526986790, 0.3774028556,
					-0.1106244044, -0.0238494650, 0.0378284555, 0.0};
	double g[10] = {0.0, 0.0, -0.0645388826, 0.0406894176, 0.4180922732, -0.7884856164, 0.4180922732,
					0.0406894176, -0.0645388826, 0.0};

	int c1, c2, resultat;

	c1 = 10; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int bio55(double *valeur, int m) /* transforme biorthogonale 5.5 d'un tableau de 2^m valeurs */
{
	double h[12] = {0.0134567095, -0.0026949669, -0.1367065847, -0.0935046974, 0.4768032658, 0.8995061097,
					0.4768032658, -0.0935046974, -0.1367065847, -0.0026949669, 0.0134567095, 0.0};
	double g[12] = {0.0, 0.0, 0.0396870883, -0.0079481086, -0.0544637885, -0.3456052820, 0.7366601814,
					-0.3456052820, -0.0544637885, -0.0079481086, 0.0396870883, 0.0};
	int c1, c2, resultat;

	c1 = 12; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio55_inv(double *valeur, int m) /* transforme biorthogonale inverse 5.5 d'un tableau de 2^m valeurs */
{
	double h[12] = {0.0, 0.0396870883, 0.0079481086, -0.0544637885, 0.3456052820, 0.7366601814, 0.3456052820,
					-0.0544637885, 0.0079481086, 0.0396870883, 0.0, 0.0};
	double g[12] = {0.0, -0.0134567095, -0.0026949669, 0.1367065847, -0.0935046974, -0.4768032658, 0.8995061097,
					-0.4768032658, -0.0935046974, 0.1367065847, -0.0026949669, -0.0134567095};

	int c1, c2, resultat;

	c1 = 12; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 16; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}

int bio68(double *valeur, int m) /* transforme biorthogonale 6.8 d'un tableau de 2^m valeurs */
{
	double h[18] = {0.0, 0.0, 0.0, 0.0144262825, 0.0144675049, -0.0787220011, -0.0403679790, 0.4178491092,
					0.7589077295, 0.4178491092, -0.0403679790, -0.0787220011, 0.0144675049, 0.0144262825,
					0.0, 0.0, 0.0, 0.0};
	double g[18] = {0.0, -0.0019088317, -0.0019142861, 0.0169906399, 0.0119345653, -0.0497329035, -0.0772631732,
					0.0940592035, 0.4207962846, -0.8259229975, 0.4207962846, 0.0940592035, -0.0772631732,
					-0.0497329035, 0.0119345653, 0.0169906399, -0.0019142861, -0.0019088317};
	int c1, c2, resultat;

	c1 = 18; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */

	resultat = transformee(valeur, m, h, g, c1); /* transforme d'un tableau de 2^m valeurs */

	return(resultat);
}

int bio68_inv(double *valeur, int m) /* transforme biorthogonale inverse 6.8 d'un tableau de 2^m valeurs */
{
	double h[18] = {0.0019088317, -0.0019142861, -0.0169906399, 0.0119345653, 0.0497329035, -0.0772631732,
					-0.0940592035, 0.4207962846, 0.8259229975, 0.4207962846, -0.0940592035, -0.0772631732,
					0.0497329035, 0.0119345653, -0.0169906399, -0.0019142861, 0.0019088317, 0.0};
	double g[18] = {0.0, 0.0, 0.0, 0.0, 0.0144262825, -0.0144675049, -0.0787220011, 0.0403679790, 0.4178491092,
					-0.7589077295, 0.4178491092, 0.0403679790, -0.0787220011, -0.0144675049, 0.0144262825,
					0.0, 0.0, 0.0};

	int c1, c2, resultat;

	c1 = 18; /* nombre de coefficients de l'ondelette = tjr un nombre pair */
	c2 = 32; /* puissance de 2 suprieure  c1 */
	
	resultat = inverse(valeur, m, h, g, c1); /* transforme inverse d'un tableau de 2^m valeurs */

	return(resultat);	
}
